"use strict";
const jbase_utils = require("./jbase_utils");
const JaPublisher = require("./amqp_utils/JaPublisher");
const JaSubscriber = require("./amqp_utils/JaSubscriber");
const Jlog = require("./jlog");
var jlog = Jlog("JaAmqp");
var amqp = require('amqp');
var publisher = require('./amqp_utils/publisher'); // 发布者
var subscriber = require('./amqp_utils/subscriber'); // 订阅者
module.exports = class JaAmqp {
    // 构造
    constructor() {
        /**
         * 就需状态
         */
        this.readyOk = false;
        this.channel = undefined;
    }
    /**
     * 初始化
     * @param cfg_or_client 配置或客户端
     * @returns 未定义
     */
    init(cfg_or_client) {
        var self = this;
        return new Promise((resolve, reject) => {
            /**
             * 创建发布者(废弃的)
             * @returns 发布者对象
             */
            self.createPublisher = () => {
                return new publisher(self.channel);
            };
            /**
             * 创建订阅者(废弃的)
             * @returns 订阅者对象
             */
            self.createSubscriber = () => {
                return new subscriber(self.channel);
            };
            if (!cfg_or_client) {
                cfg_or_client = {};
            }
            if (cfg_or_client.on) {
                // 使用通道
                self.channel = cfg_or_client;
                resolve();
            }
            else {
                // 默认配置
                var mqDefCfg = {
                    "host": "127.0.0.1",
                    "port": 5672,
                    "login": "guest",
                    "password": "guest",
                    "connectionTimeout": 10000, // 连接超时
                    "authMechanism": "AMQPLAIN", // 认证方式
                    "heartbeat": 10, // 心跳(秒)
                    "vhost": "/", // 默认vhost
                    "noDelay": true, // tcp优化
                    "reconnectExponentialLimit": 999999999, // 重连限制
                    "ssl": {
                        "enabled": false
                    }
                };
                // 合并用户配置
                jbase_utils.unionObj(mqDefCfg, cfg_or_client);
                // 建立通道
                self.channel = amqp.createConnection(mqDefCfg);
                // 等待就需
                self.channel.on('ready', function () {
                    // 只允许进入一次
                    if (!self.readyOk) {
                        self.readyOk = true;
                        if (jlog.jlogEnable)
                            jlog.info("amqp init ok");
                        resolve();
                    }
                });
                // 错误处理
                self.channel.on('error', function (err) {
                    if (jlog.jlogEnable)
                        jlog.error("error:", err);
                });
            }
        }); // End Promise
    }
    /**
     * 获取Amqp客户端通道
     */
    getChannel() {
        return this.channel;
    }
    /**
     * 创建发布者
     * @returns 发布者对象
     */
    createPub() {
        return new JaPublisher(this.channel);
    }
    /**
     * 创建订阅者
     * @returns 订阅者对象
     */
    createSub() {
        return new JaSubscriber(this.channel);
    }
};
