import jbase_utils = require('./jbase_utils');
import jdocode = require('./jdocode');
import jtype_utils = require('./jtype_utils');

import JaPublisher = require('./amqp_utils/JaPublisher');
import JaSubscriber = require('./amqp_utils/JaSubscriber');

import Jlog = require('./jlog');
var jlog = Jlog("JaAmqp");

var amqp = require('amqp');

var publisher = require('./amqp_utils/publisher'); // 发布者
var subscriber = require('./amqp_utils/subscriber'); // 订阅者



/**
 * MQ工具类
 */
export = class JaAmqp {

    /**
     * 就需状态
     */
    private readyOk = false;
    private channel = <any>undefined;

    // 构造
    constructor() {

    }

    /**
     * 初始化
     * @param cfg_or_client 配置或客户端
     * @returns 未定义
     */
    init(cfg_or_client: any) {
        var self = this;
        return new Promise<void>((resolve, reject) => { // Begin Promise

            /**
             * 创建发布者(废弃的)
             * @returns 发布者对象
             */
            (<any>self).createPublisher = () => {
                return new publisher(self.channel);
            }

            /**
             * 创建订阅者(废弃的)
             * @returns 订阅者对象
             */
            (<any>self).createSubscriber = () => {
                return new subscriber(self.channel);
            }

            if (!cfg_or_client) {
                cfg_or_client = {};
            }

            if ((<any>cfg_or_client).on) {
                // 使用通道
                self.channel = <any>cfg_or_client;
                resolve();
            }
            else {

                // 默认配置
                var mqDefCfg = {
                    "host": "127.0.0.1",
                    "port": 5672,
                    "login": "guest",
                    "password": "guest",
                    "connectionTimeout": 10000, // 连接超时
                    "authMechanism": "AMQPLAIN", // 认证方式
                    "heartbeat": 10, // 心跳(秒)
                    "vhost": "/", // 默认vhost
                    "noDelay": true, // tcp优化
                    "reconnectExponentialLimit": 999999999, // 重连限制
                    "ssl": {
                        "enabled": false
                    }
                }

                // 合并用户配置
                jbase_utils.unionObj(mqDefCfg, cfg_or_client)

                // 建立通道
                self.channel = amqp.createConnection(<any>mqDefCfg);

                // 等待就需
                self.channel.on('ready', function () {
                    // 只允许进入一次
                    if (!self.readyOk) {
                        self.readyOk = true;
                        if (jlog.jlogEnable) jlog.info("amqp init ok");
                        resolve();
                    }
                });

                // 错误处理
                self.channel.on('error', function (err: any) {
                    if (jlog.jlogEnable) jlog.error("error:", err)
                });
            }

        }); // End Promise
    }

    /**
     * 获取Amqp客户端通道
     */
    getChannel() {
        return this.channel;
    }

    /**
     * 创建发布者
     * @returns 发布者对象
     */
    createPub() {
        return new JaPublisher(this.channel);
    }

    /**
     * 创建订阅者
     * @returns 订阅者对象
     */
    createSub() {
        return new JaSubscriber(this.channel);
    }


}

