"use strict";
/*

// 表单文件
yield jhttp_utils.postMultipart("http://127.0.0.1:666/demo/post_form_file", {
    p1: "123",
    p2: "abc",
    file1: {
        value: fs.createReadStream("D:\\J-USER\\desktop7\\rarlinux-x64-5.6.1.tar.gz"),
        options: {
            filename: 'topsecret.jpg'
        }
    }
});

*/
const fs = require("fs");
const jbase_utils = require("./jbase_utils");
const JERR_S = require("./jerror_type");
const jret_utils = require("./jret_utils");
const jco = require("./jco");
const Jlog = require("./jlog");
var jlog = Jlog("jhttp_utils");
const request = require("request");
const http = require("http");
const https = require("http");
// 设置全局代理最大链接数为无限制
http.globalAgent.maxSockets = Infinity;
https.globalAgent.maxSockets = Infinity;
/**
 * 合并多级对象
 * @param dst 目标
 * @param obj 源对象
 * @returns 无
 */
function unionMyObj(dst, obj) {
    if (!dst || !obj)
        return;
    for (var a in obj) {
        if ((a === "headers" || a === "pool") && Object.prototype.toString.call(obj[a]) === "[object Object]") {
            if (Object.prototype.toString.call(dst[a]) !== "[object Object]") {
                dst[a] = {};
            }
            unionMyObj(dst[a], obj[a]); //递归遍历
        }
        else {
            dst[a] = obj[a];
        }
    }
}
module.exports = class JaHttpUtils {
    // 构造
    constructor() {
        /**
         * 内部对象
         */
        this.innerObj = {
            /**
             * 默认请求配置项
             */
            defReqOpt: {
                timeout: 70000,
                // headers: {
                //     "User-Agent": 'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0)'
                // }
            },
            /**
             * 前置过滤器
             */
            firstFilter: null,
            /**
             * 后置过滤器
             */
            lastFilter: null,
            /**
             * 结果过滤器
             */
            retFilter: null,
            /**
             * 错误回调处理
             */
            errCbFun: null,
            /**
             * 下载的句柄号
            */
            downFdCur: 1,
            /**
             * 下载http文件(错误将放入返回的err中,不会抛出)
             * @param url 下载地址
             * @param localFile 本地地址,省略为临时路径或者使用回调函数确定
             * @param opt 选项(可省)
             * @param cbFun 回调函数(可省)用于文件头的处理,返回:false表示阻止继续下载,返回路径表示下载到指定位置
             * @returns 返回文件信息;
             */
            downloadFileEx: function (url, localFile, reqOpt, cbFun) {
                var iobj = this;
                // 唯一ID
                var juuid = jbase_utils.getJstamp();
                reqOpt.headers.jstamp = juuid;
                // 日志
                if (jlog.jlogEnable)
                    jlog.info(`[${juuid}]downloadFileEx:url=${url},localFile=${localFile},opt=`, reqOpt);
                // 临时路径
                var localFile_t;
                // 返回对象
                var retObj = {
                    dwFd: iobj.downFdCur++,
                    errno: 0,
                    fileSize: -1,
                    fileSerSize: -1,
                    timeStart: Date.now()
                };
                // 下载对象
                var dwObj = {
                    dwFd: retObj.dwFd,
                    current: 0,
                    total: -1,
                    about: null,
                };
                // 返回Promise
                return new Promise((resolve, reject) => {
                    // 创建文件对象
                    var fd = null;
                    var curState = 0; // 状态,0=未决,1=成功,2=失败
                    // 结束
                    var isDoResolve = false;
                    var doResolve = (errCode, curErr) => {
                        // 错误标记
                        if (errCode !== JERR_S.OK) {
                            curState = 2;
                        }
                        // 关闭输出文件
                        if (fd) {
                            fs.close(fd, (err) => { });
                            fd = null;
                        }
                        // 防止反复处理
                        if (isDoResolve)
                            return;
                        isDoResolve = true;
                        // 错误码
                        retObj.errno = errCode;
                        retObj.code = JERR_S.getName(errCode);
                        if (errCode !== JERR_S.OK) {
                            if (curErr) {
                                retObj.err = curErr;
                            }
                            else {
                                retObj.err = JERR_S.newError(errCode);
                            }
                            retObj.err.jerrno = retObj.errno;
                        }
                        //写入时间
                        retObj.totalTime = Date.now() - retObj.timeStart;
                        // 返回
                        if (jlog.jlogEnable) {
                            jlog.info(`[${juuid}]downloadFileEx:resolve=`, retObj);
                        }
                        resolve(retObj);
                    };
                    // 创建请求
                    try {
                        var writeBusy = false;
                        if (iobj.firstFilter) {
                            var newUrl = iobj.firstFilter(1, url, reqOpt);
                            if (typeof (newUrl) === 'string')
                                url = newUrl;
                        }
                        // 随机url参数
                        if (reqOpt.randomUrl) {
                            if (!reqOpt.qs)
                                reqOpt.qs = {};
                            reqOpt.qs.jrnd = Math.random();
                        }
                        // 请求
                        var req = request(url, reqOpt);
                        // 请求成功
                        req.on('response', (response) => {
                            // 当前大小
                            dwObj.current = 0;
                            // 解析文件大小
                            dwObj.total = -2;
                            try {
                                dwObj.total = parseInt(response.headers['content-length'] || "-2", 10);
                            }
                            catch (error) {
                            }
                            if (!Number.isInteger(dwObj.total))
                                dwObj.total = -2;
                            retObj.fileSerSize = dwObj.total;
                            // 用户检查
                            var userChk = true;
                            if (cbFun) {
                                try {
                                    // 放入停止函数
                                    dwObj.about = () => {
                                        // 用户取消
                                        doResolve(JERR_S.USER_ABOUT, new Error("user about"));
                                        response.destroy();
                                        req.destroy();
                                    };
                                    // 回调用户函数
                                    userChk = cbFun(response, dwObj);
                                    if (typeof (userChk) === "string") {
                                        localFile = userChk;
                                    }
                                }
                                catch (errUser) {
                                    userChk = false;
                                }
                            }
                            // 临时路径
                            localFile_t = localFile + "." + jbase_utils.getRndInt(0, 100000000) + ".tmp";
                            retObj.statusCode = response.statusCode;
                            retObj.contentType = response.headers['content-type'];
                            // 加入http头
                            if (reqOpt.needResHeader)
                                retObj.resHeader = response.headers;
                            // 返回
                            if (jlog.jlogEnable) {
                                jlog.trace(`[${juuid}]downloadFileEx:statusCode=${retObj.statusCode},contentType=${retObj.contentType}`);
                            }
                            if (userChk !== false && retObj.statusCode === 200) {
                                // 成功,先暂停接收
                                response.pause();
                                // 创建文件
                                fs.open(localFile_t, "w", (err, res) => {
                                    if (err) {
                                        // 文件系统错误
                                        doResolve(JERR_S.FILE_ERR, err);
                                        // 停止网络流
                                        response.destroy();
                                        req.destroy();
                                    }
                                    else {
                                        // 句柄
                                        fd = res;
                                        // 注册数据接收函数
                                        response.on('data', (data) => {
                                            response.pause();
                                            if (fd) {
                                                writeBusy = true;
                                                dwObj.current += data.length;
                                                fs.write(fd, data, 0, data.length, (err, res) => {
                                                    response.resume();
                                                    // 取消忙标记
                                                    if (writeBusy === true) {
                                                        writeBusy = false;
                                                    }
                                                    // 如果是函数则执行
                                                    else {
                                                        writeBusy();
                                                    }
                                                    if (err) {
                                                        // 文件系统错误
                                                        doResolve(JERR_S.FILE_ERR, err);
                                                        // 停止网络流
                                                        response.destroy();
                                                        req.destroy();
                                                    }
                                                });
                                            }
                                            else {
                                                // 停止网络流
                                                response.destroy();
                                                req.destroy();
                                                // 文件错误
                                                doResolve(JERR_S.FILE_ERR, new Error("no fd"));
                                            }
                                        });
                                        // 注册数据完成函数
                                        response.on('end', () => {
                                            if (fd) {
                                                var dataEndDo = function () {
                                                    // 成功
                                                    if (curState === 0)
                                                        curState = 1;
                                                    //结束流
                                                    fs.close(fd, (err) => {
                                                        if (err) {
                                                            // 文件错误
                                                            doResolve(JERR_S.FILE_ERR, err);
                                                            // 删除文件
                                                            fs.unlink(localFile_t, () => { });
                                                        }
                                                        else if (curState === 1) {
                                                            // 判断文件是否是需要的大小
                                                            if (reqOpt.chkSerSize && retObj.fileSerSize > 0 && dwObj.current < retObj.fileSerSize) {
                                                                // 网络错误
                                                                doResolve(JERR_S.HTTP_NET_ERR, new Error("fileSize < fileSerSize"));
                                                                // 删除文件
                                                                fs.unlink(localFile_t, () => { });
                                                            }
                                                            else {
                                                                // 重命名
                                                                fs.rename(localFile_t, localFile ? localFile : "localFile.bin", (error) => {
                                                                    if (error) {
                                                                        // 文件系统错误
                                                                        doResolve(JERR_S.FILE_ERR, error);
                                                                        // 删除文件
                                                                        fs.unlink(localFile_t, () => { });
                                                                    }
                                                                    else {
                                                                        // 写入文件信息
                                                                        retObj.fileSize = dwObj.current;
                                                                        // 后置过滤
                                                                        if (iobj.lastFilter) {
                                                                            var retFilterObj = iobj.lastFilter(1, url, reqOpt, retObj, response);
                                                                            if (retFilterObj instanceof Error) {
                                                                                return doResolve(JERR_S.HTTP_SER_ERR, retFilterObj);
                                                                            }
                                                                        }
                                                                        // 成功
                                                                        return doResolve(JERR_S.OK);
                                                                    }
                                                                });
                                                            }
                                                        }
                                                    });
                                                    // 清除句柄
                                                    fd = null;
                                                };
                                                if (writeBusy === true) {
                                                    (new Promise((resolve, reject) => {
                                                        writeBusy = resolve;
                                                    })).then(function () {
                                                        dataEndDo();
                                                    });
                                                }
                                                else {
                                                    dataEndDo();
                                                }
                                            }
                                            else {
                                                // 文件错误
                                                doResolve(JERR_S.FILE_ERR, new Error("no fd"));
                                            }
                                        });
                                        // 继续流接收
                                        response.resume();
                                    }
                                });
                            }
                            else if (userChk === false) {
                                // 用户取消
                                doResolve(JERR_S.USER_ABOUT, new Error("user cancel"));
                                // 停止网络流
                                response.destroy();
                                req.destroy();
                            }
                            else {
                                // 错误处理
                                var errBodyArr = [];
                                var bodyLen = 0;
                                var retErr = new Error(retObj.statusCode + "");
                                // 接收错误信息
                                response.on('data', (data) => {
                                    // 错误信息
                                    if (bodyLen < 4096) {
                                        bodyLen += data.length;
                                        errBodyArr.push(data);
                                    }
                                    else {
                                        // 太长了,提前结束,将会提前激发end事件
                                        response.destroy();
                                    }
                                });
                                // 结束接收
                                response.on('end', () => {
                                    // 合并错误信息
                                    retErr.bodyBuffer = Buffer.concat(errBodyArr);
                                    // 返回
                                    var errStr = "unknown";
                                    try {
                                        errStr = retErr.bodyBuffer.toString();
                                        retErr.body = errStr;
                                    }
                                    catch (error) {
                                    }
                                    if (jlog.jlogEnable) {
                                        jlog.error(`[${juuid}]downloadFileEx Error:statusCode=${retObj.statusCode},contentType=${retObj.contentType},body=${errStr}`);
                                    }
                                    if (retObj.statusCode === 578) {
                                        // 连接错误
                                        doResolve(JERR_S.HTTP_NET_ERR);
                                    }
                                    else {
                                        // 服务器错误
                                        doResolve(JERR_S.HTTP_SER_ERR, retErr);
                                    }
                                    // 停止网络流
                                    response.destroy();
                                    req.destroy();
                                });
                                // 继续流接收
                                response.resume();
                            }
                        });
                        // 错误事件处理
                        req.on('error', (error) => {
                            // 连接错误
                            doResolve(JERR_S.HTTP_NET_ERR, error);
                        });
                    }
                    catch (error) {
                        // 参数错误
                        doResolve(JERR_S.PARAM_ERROR, error);
                    }
                }); // End Promise
            },
            /**
             * 发送请求
             * @param url 地址
             * @param reqOpt 选项
             * @returns 返回内容
             */
            sendDataEx: function (url, reqOpt) {
                var iobj = this;
                // 唯一ID
                var juuid = jbase_utils.getJstamp();
                reqOpt.juuid = juuid;
                // 日志
                if (jlog.jlogEnable)
                    jlog.info(`[${juuid}]sendData:url=${url},reqOpt=`, reqOpt);
                // 返回Promise
                return new Promise((resolve, reject) => {
                    try {
                        // 前置过滤器
                        if (iobj.firstFilter) {
                            var newUrl = iobj.firstFilter(2, url, reqOpt);
                            if (typeof (newUrl) === 'string')
                                url = newUrl;
                        }
                        // 随机url参数
                        if (reqOpt.randomUrl) {
                            if (!reqOpt.qs)
                                reqOpt.qs = {};
                            reqOpt.qs.jrnd = Math.random();
                        }
                        // 是否直接使用Buffer返回
                        var isBufferRet = false;
                        if (reqOpt.encoding === "buffer") {
                            reqOpt.encoding = null;
                            // 不需要指定应答类型
                            if (reqOpt.headers)
                                delete reqOpt.headers.Accept;
                            isBufferRet = true;
                            reqOpt.stdBody = false;
                        }
                        // 拦截器参数
                        var flObj = {
                            uuid: juuid,
                            url: url,
                            xhr: null,
                            dataObj: null,
                            reqOpt: reqOpt,
                            retObj: null,
                            response: null
                        };
                        // 发送请求
                        request(url, reqOpt, (error, response, resBody) => {
                            try {
                                if (!response)
                                    response = {};
                                flObj.response = response; // 返回体
                                flObj.dataObj = resBody; // 结果数据
                                if (error) {
                                    if (jlog.jlogEnable) {
                                        jlog.info(`[${juuid}]sendData:Err=`, error, response.statusCode);
                                    }
                                    // 结果过滤器
                                    if (iobj.retFilter) {
                                        iobj.retFilter(error, flObj);
                                    }
                                    if (response.statusCode === 578) {
                                        // 抛错
                                        return reject(JERR_S.newError(JERR_S.HTTP_NET_ERR, error));
                                    }
                                    else {
                                        return reject(JERR_S.newError(error.code ? JERR_S.HTTP_NET_ERR : JERR_S.PARAM_ERROR, error));
                                    }
                                }
                                else if (response.statusCode === 200) {
                                    // 返回体类型
                                    var retObj = null;
                                    // json字符串
                                    if (!isBufferRet) {
                                        try {
                                            retObj = JSON.parse(resBody);
                                        }
                                        catch (err) {
                                            if (jlog.jlogEnable) {
                                                jlog.error(`[${juuid}]sendData:retObj JSON parse error!`);
                                            }
                                            // 结果过滤器
                                            if (iobj.retFilter) {
                                                iobj.retFilter(err, flObj);
                                            }
                                            return reject(JERR_S.newError(JERR_S.HTTP_SER_ERR, err));
                                        }
                                    }
                                    // 没有数据,填充原始数据
                                    if (!retObj) {
                                        retObj = { body: resBody };
                                        if (jlog.jlogEnable) {
                                            jlog.info(`[${juuid}]sendData:retObj length=${resBody.length}`);
                                        }
                                    }
                                    else if (jlog.jlogEnable) {
                                        jlog.info(`[${juuid}]sendData:retObj=`, retObj);
                                    }
                                    // 结果过滤器
                                    if (iobj.retFilter) {
                                        flObj.retObj = retObj;
                                        iobj.retFilter(null, flObj);
                                    }
                                    // 后置过滤
                                    if (iobj.lastFilter) {
                                        var retFilterObj = iobj.lastFilter(2, url, reqOpt, retObj, response);
                                        if (retFilterObj instanceof Error) {
                                            var errObj = JERR_S.newError(JERR_S.HTTP_SER_ERR);
                                            errObj.statusCode = response.statusCode;
                                            errObj.body = retObj;
                                            errObj.err = retFilterObj;
                                            return reject(errObj);
                                        }
                                    }
                                    // 标准返回体
                                    if (reqOpt.stdBody) {
                                        var retStdBody = jret_utils.getStdBody(retObj, null);
                                        if (retStdBody) {
                                            retObj = retStdBody;
                                        }
                                        else {
                                            var errObj = JERR_S.newError(JERR_S.HTTP_SER_ERR, retObj.msg);
                                            errObj.statusCode = response.statusCode;
                                            errObj.body = retObj;
                                            errObj.errCode = retObj.code;
                                            return reject(errObj);
                                        }
                                    }
                                    // 加入http头
                                    if (reqOpt.needResHeader) {
                                        retObj.resHeader = response.headers;
                                        jlog.info(`[${juuid}]sendData:resHeader=`, retObj.resHeader);
                                    }
                                    // 返回数据
                                    return resolve(retObj);
                                }
                                else {
                                    // 服务器错误
                                    var errBody = "";
                                    try {
                                        errBody = resBody.toString().substr(0, 2048);
                                    }
                                    catch (error) {
                                    }
                                    if (jlog.jlogEnable) {
                                        jlog.error(`[${juuid}]sendData:errCode=`, response.statusCode, ",body=", errBody);
                                    }
                                    // 抛错
                                    var errObj = JERR_S.newError(JERR_S.HTTP_SER_ERR);
                                    errObj.statusCode = response.statusCode;
                                    errObj.body = errBody;
                                    // 结果过滤器
                                    if (iobj.retFilter) {
                                        iobj.retFilter(errObj, flObj);
                                    }
                                    return reject(errObj);
                                }
                            }
                            catch (error) {
                                if (jlog.jlogEnable) {
                                    jlog.error(`[${juuid}]sendData:Err=`, error);
                                }
                                // 解析错误
                                var errObj = JERR_S.newError(JERR_S.RET_FORMAT_ERR, error);
                                errObj.statusCode = response.statusCode;
                                // 结果过滤器
                                if (iobj.retFilter) {
                                    iobj.retFilter(errObj, flObj);
                                }
                                return reject(errObj);
                            }
                        });
                    }
                    catch (error) {
                        // 参数错误
                        return reject(JERR_S.newError(JERR_S.PARAM_ERROR, error));
                    }
                });
            }
        };
    }
    /**
     * 下载http文件
     * @param url 下载地址
     * @param localFile 本地地址
     * @param reqOpt 选项(可省)
     * @param cbFun 回调函数(可省),用于文件头的处理,返回:false表示阻止继续下载,返回路径表示下载到指定位置
     * @returns 返回文件信息{err:错误码,fileSize:文件大小,statusCode:http状态码,contentType:文件类型};
     */
    downloadFile(url, localFile, reqOpt, cbFun) {
        var iobj = this.innerObj;
        var opt = {
            method: 'GET',
            encoding: null,
            gzip: false,
            headers: {
                'Accept': '*/*'
            }
        };
        // 合并
        unionMyObj(opt, iobj.defReqOpt);
        unionMyObj(opt, reqOpt);
        // 处理错误重试
        if (opt.tryErrCount || iobj.errCbFun) {
            return jco(function* () {
                var ret;
                // 超时重试间隔
                var tryErrInterval = opt.tryErrInterval ? opt.tryErrInterval : 5000;
                var tryErrNet = (opt.tryErrNet === undefined) ? false : opt.tryErrNet;
                // 计算重试次数
                var tryErrCount = 1;
                if (opt.tryErrCount) {
                    if (opt.tryErrCount > 0) {
                        tryErrCount = opt.tryErrCount + 1;
                    }
                    else {
                        tryErrCount = opt.tryErrCount;
                    }
                }
                var timeStart;
                var mustRetry;
                while (true) {
                    timeStart = Date.now();
                    ret = yield iobj.downloadFileEx(url, localFile, opt, cbFun);
                    if (ret.errno === JERR_S.OK) {
                        return ret; // 成功
                    }
                    // 错误处理
                    mustRetry = undefined;
                    if (iobj.errCbFun) {
                        mustRetry = yield iobj.errCbFun(1, url, ret.errno, opt, ret, ret.statusCode, ret.err);
                    }
                    if (typeof (mustRetry) === "number") {
                        // 延时重试
                        var leftSleep = mustRetry - (Date.now() - timeStart);
                        if (leftSleep > 0) {
                            yield jbase_utils.sleep(leftSleep);
                        }
                    }
                    else if (((JERR_S.HTTP_NET_ERR === ret.errno) && (tryErrNet)) // 网络错误
                        || ((JERR_S.FILE_ERR === ret.errno) && (opt.tryErrFile)) //文件系统错误
                        || ((JERR_S.HTTP_SER_ERR === ret.errno) && (opt.tryErrServer)) // 服务器错误
                    ) {
                        // 出现错误
                        if (--tryErrCount === 0) {
                            if (opt.noError) {
                                return ret; // 返回
                            }
                            throw ret.err;
                        }
                        // 延时重试
                        var leftSleep = tryErrInterval - (Date.now() - timeStart);
                        if (leftSleep > 0) {
                            yield jbase_utils.sleep(leftSleep);
                        }
                    }
                    else {
                        // 含有错误
                        if (opt.noError) {
                            return ret; // 返回
                        }
                        throw ret.err;
                    }
                }
            });
        }
        else {
            // 无选项
            return iobj.downloadFileEx(url, localFile, opt, cbFun).then((ret) => {
                // 含有错误,抛出
                if (ret.err) {
                    if (opt && opt.noError) {
                        // 返回含错误的数据
                        return ret;
                    }
                    // 抛出错误
                    throw ret.err;
                }
                else {
                    // 返回正确数据
                    return ret;
                }
            });
        }
    }
    /**
     * 发送请求,超时控制
     * @param url 地址
     * @param optBase 基础选项
     * @param optUser 用户选项,将会和基础选项合并
     * @returns 返回内容
     */
    sendEx(url, optBase, optUser) {
        var iobj = this.innerObj;
        var opt = {
            method: 'POST',
            encoding: null,
            gzip: true,
            headers: {
                'Accept': '*/*',
                'Cache-Control': 'no-cache'
            }
        };
        // 合并用户选项
        unionMyObj(opt, optBase);
        unionMyObj(opt, iobj.defReqOpt);
        unionMyObj(opt, optUser);
        // 处理错误重试
        if (opt.tryErrCount || iobj.errCbFun) {
            return jco(function* () {
                var ret;
                // 超时重试间隔
                var tryErrInterval = opt.tryErrInterval ? opt.tryErrInterval : 5000;
                var tryErrNet = (opt.tryErrNet === undefined) ? false : opt.tryErrNet;
                // 计算重试次数
                var tryErrCount = 1;
                if (opt.tryErrCount) {
                    if (opt.tryErrCount > 0) {
                        tryErrCount = opt.tryErrCount + 1;
                    }
                    else {
                        tryErrCount = opt.tryErrCount;
                    }
                }
                var mustRetry;
                var timeStart = 0;
                while (true) {
                    try {
                        timeStart = Date.now();
                        ret = yield iobj.sendDataEx(url, opt);
                        return ret; // 成功
                    }
                    catch (error) {
                        // 错误处理
                        mustRetry = undefined;
                        if (iobj.errCbFun) {
                            mustRetry = yield iobj.errCbFun(2, url, error.errno, opt, error.body, error.statusCode, error);
                        }
                        if (typeof (mustRetry) === "number") {
                            // 延时重试
                            var leftSleep = mustRetry - (Date.now() - timeStart);
                            if (leftSleep > 0) {
                                yield jbase_utils.sleep(leftSleep);
                            }
                        }
                        else if (((JERR_S.HTTP_NET_ERR === error.errno) && (tryErrNet)) // 网络错误
                            || ((JERR_S.FILE_ERR === error.errno) && (opt.tryErrFile)) //文件系统错误
                            || ((JERR_S.HTTP_SER_ERR === error.errno) && (opt.tryErrServer)) // 服务器错误
                        ) {
                            if (--tryErrCount === 0) {
                                throw error; // 直接抛出
                            }
                            // 延时重试
                            var leftSleep = tryErrInterval - (Date.now() - timeStart);
                            if (leftSleep > 0) {
                                yield jbase_utils.sleep(leftSleep);
                            }
                        }
                        else {
                            throw error; // 直接抛出
                        }
                    }
                }
            });
        }
        else {
            return iobj.sendDataEx(url, opt);
        }
    }
    /**
     * GET请求数据
     * @param url 地址
     * @param obj 提交的对象(将会转换到url段)
     * @param opt 选项
     * @returns 返回内容
     */
    getData(url, obj, opt) {
        var self = this;
        return self.sendEx(url, {
            method: "GET",
            headers: {
                "Accept": "application/json"
            },
            qs: obj
        }, opt);
    }
    /**
     * 发送Json数据(application/json)
     * @param method 动作"POST"|"PUT"|"DELETE"或其他
     * @param url 地址
     * @param obj 提交的对象
     * @param opt 选项
     * @returns 返回内容
     */
    sendJson(method, url, obj, opt) {
        var self = this;
        return self.sendEx(url, {
            method: method.toUpperCase(),
            headers: {
                "Accept": "application/json",
                "Content-Type": "application/json"
            },
            body: (typeof (obj) === "string") ? obj : JSON.stringify(obj || {})
        }, opt);
    }
    /**
     * 发送表单数据(application/x-www-form-urlencoded)
     * @param method 动作"POST"|"GET"|"PUT"|"DELETE"或其他
     * @param url 地址
     * @param obj 提交的对象
     * @param opt 选项
     * @returns 返回内容
     */
    sendWww(method, url, obj, opt) {
        var self = this;
        return self.sendEx(url, {
            method: method.toUpperCase(),
            headers: {
                "Accept": "application/json"
            },
            form: obj || {}
        }, opt);
    }
    /**
     * 发送Multipart表单数据(multipart/form-data)
     * @param method 动作"POST"|"PUT"|"DELETE"或其他
     * @param url 地址
     * @param obj 提交的对象
     * @param opt 选项
     * @returns 返回内容
     */
    sendMultipart(method, url, obj, opt) {
        var self = this;
        return self.sendEx(url, {
            method: method.toUpperCase(),
            headers: {
                "Accept": "application/json"
            },
            formData: obj || {}
        }, opt);
    }
    /**
     * Post Json数据(application/json)
     * @param url 地址
     * @param obj 提交的对象
     * @param opt 选项
     * @returns 返回内容
     */
    postJson(url, obj, opt) {
        var self = this;
        return self.sendJson("POST", url, obj, opt);
    }
    /**
     * Post 表单数据(application/x-www-form-urlencoded)
     * @param url 地址
     * @param obj 提交的对象
     * @param opt 选项
     * @returns 返回内容
     */
    postWww(url, obj, opt) {
        var self = this;
        return self.sendWww("POST", url, obj, opt);
    }
    /**
     * Post Multipart表单数据(multipart/form-data)
     * @param url 地址
     * @param obj 提交的对象
     * @param opt 选项
     * @returns 返回内容
     */
    postMultipart(url, obj, opt) {
        var self = this;
        return self.sendMultipart("POST", url, obj, opt);
    }
    /**
     * 修改默认超时时间
     * @param timeOut 超时时间(ms),为0是获取当前超时
     * @returns 变更后的超时时间
     */
    setDefTimeout(timeOut) {
        var iobj = this.innerObj;
        if (timeOut)
            iobj.defReqOpt.timeout = timeOut;
        return iobj.defReqOpt.timeout;
    }
    /**
     * 修改默认配置
     * @param defOpt 默认配置
     * @returns 无
     */
    setDefOption(defOpt) {
        var iobj = this.innerObj;
        iobj.defReqOpt = {};
        jbase_utils.unionObj(iobj.defReqOpt, defOpt);
    }
    /**
     * 设置前置过滤器
     * @param firstFilter 前置过滤器
     * @returns 无
     */
    setFirstFilter(firstFilter) {
        var iobj = this.innerObj;
        iobj.firstFilter = firstFilter;
    }
    /**
     * 设置后置过滤器
     * @param lastFilter 后置过滤器
     * @returns 无
     */
    setLastFilter(lastFilter) {
        var iobj = this.innerObj;
        iobj.lastFilter = lastFilter;
    }
    /**
     * 设置结果过滤器
     * @param retFilter 结果过滤器
     * @returns 无
     */
    setRetFilter(retFilter) {
        var iobj = this.innerObj;
        iobj.retFilter = retFilter;
    }
    /**
     * 设置错误回调函数,支持Promise
     * @param cbFun 回调函数
     */
    setErrCbFun(cbFun) {
        var iobj = this.innerObj;
        iobj.errCbFun = cbFun;
    }
};
