
import jbase_utils = require('./jbase_utils');

var jwt = require('jsonwebtoken');


/**
 * 生成BasicAuth验证字符串
 * @param userName 用户名
 * @param passWord 密码
 * @param tokenName token前段,省略="Basic",""=无前端
 * @returns BasicAuth验证字符串
 */
export function makeBasicAuth(userName: string, passWord: string, tokenName?: string) {
    if (!tokenName) tokenName = "Basic";
    if (tokenName) tokenName = tokenName + " ";
    if (<any>Buffer.alloc) {
        return tokenName + Buffer.from(userName + ":" + passWord).toString("base64");
    }
    else {
        return tokenName + new Buffer(userName + ":" + passWord).toString("base64");
    }
}

/**
 * 获取BasicAuth对象
 * @param authorization Authorization头字符串
 * @returns BasicAuth对象
 */
export function getBasicAuth(authorization: string) {
    try {
        // 检出加密段
        if (authorization.substr(5, 1) === " ") {
            authorization = authorization.substr(6);
        }

        // 解码
        var baseStr = <String><any>null;
        if (<any>Buffer.alloc) {
            baseStr = Buffer.from(authorization, 'base64').toString();
        }
        else {
            baseStr = new Buffer(authorization, 'base64').toString();
        }

        //拆出密码
        var spIndex = baseStr.indexOf(":");
        if (spIndex >= 0) {
            return {
                userName: baseStr.substr(0, spIndex),
                passWord: baseStr.substr(spIndex + 1)
            }
        }
    } catch (error) {
    }
    return null;
}

/**
 * 制作认证JwtToken
 * @param obj 对象
 * @param jwtKey 密钥
 * @param tokenName token前段,省略="Bearer",""=无前端
 * @returns TokenAuth字符串
 */
export function makeJwtTokenStr(obj: any, jwtKey: string, tokenName?: string) {
    var token = jwt.sign(obj, jwtKey);
    if (!tokenName) tokenName = "Bearer";
    if (tokenName) tokenName = tokenName + " ";
    return tokenName + token;
}

/**
 * 获取JwtToken数据
 * @param authorization Authorization头JwtToken字符串
 * @param jwtKey 密钥
 * @returns 对象
 */
export function getJwtTokenObj(authorization: string, jwtKey: string) {
    try {
        // 检出加密段
        if (authorization.substr(6, 1) === " ") {
            authorization = authorization.substr(7);
        }
        return jwt.verify(authorization, jwtKey);
    } catch (err) {
    }
    return null;
}
