/* 
    该模块为基础功能模块
*/


import Jlog = require('./jlog');
var jlog = Jlog("jbase_utils");

import jco = require('./jco');
import jtype_utils = require('./jtype_utils');

import isYieldFun = jtype_utils.isYieldFun;

/**
 * 延时ms
 * @param timeMs 延时ms
 * @returns 无
 */
export function sleep(timeMs: number) {
    return new Promise((resolve, reject) => { // Begin Promise
        setTimeout(resolve, timeMs);
    }); // End Promise
}

// 等待对象
var waitObj: any = {};

/**
 * 序列执行:yield seqDo("aaa",function*(){内容});
 * @param seqName 等待名称,一样的名字将会顺序执行
 * @param processFun 执行函数,支持Promise:function*(){}
 * @returns 无
 */
export function seqDo(seqName: Function | string, processFun: any) {
    return jco(function* (): any {

        var waitName: string = "";

        // 判断互斥对象
        if (typeof (seqName) === "function") {
            waitName = seqName.name;
        }
        if (typeof (seqName) !== "string") {
            waitName = seqName + "_def";
        }
        else {
            waitName = seqName;
        }

        // 创建空白Promise
        var myResolve = () => { };
        var myPro = new Promise<any>(function (resolve, reject) {   // Begin Promise
            myResolve = <any>resolve; // 取出已决函数
        });

        // 存在任务
        if (waitObj[waitName]) {
            // 等待任务完成
            var wait_t = waitObj[waitName]; // 取出上个任务
            waitObj[waitName] = myPro; // 换成自己的任务
            yield wait_t; // 等待上个任务完成
            wait_t = null;
        }
        else {
            waitObj[waitName] = myPro; // 换成自己的任务
        }

        // 处理任务
        var jret = null;
        try {
            jret = yield processFun();
            myResolve(); // 已决
        } catch (error) {
            myResolve(); // 已决
            // if (jlog.jlogEnable) jlog.error("seqDo[" + waitName + "]", error);
            throw error;
        }

        return jret;
    });
}

/**
 * 模板文本替换
 * @param temStr 模板文本,如:"恭喜{userName}获得{money}元话费"
 * @param inParam 对象
 * @returns 替换后的文本
 */
export function getTemplateStr(temStr: string, inParam: any) {
    for (var curName in inParam) {
        if (typeof (inParam[curName]) === "string") {
            temStr = temStr.replace("{" + curName + "}", inParam[curName]);
        }
    }
    return temStr;
}

/**
 * 信号等待函数,返回true时将跳过该段的合并
 */
export type TypeUnionObjCbFun = (key: string, obj: any, dst: any) => any;

/**
 * 合并多级对象
 * @param dst 目标
 * @param obj 源对象
 * @param layers 深复制层数(0~N,0=表层赋值,省略或-1为无限层)
 * @param doFun 排除的内容或者排除处理函数
 * @returns 无
 */
export function unionObj(dst: any, obj?: any, layers?: number | string[] | TypeUnionObjCbFun, doFun?: string[] | TypeUnionObjCbFun): any {
    if (!dst) return;

    // 无源数据
    if (!obj) {
        // 兼容深复制
        if (arguments.length === 1) {
            console.error("warn:unionObj obj is empty!");
            return JSON.parse(JSON.stringify(dst));
        }
        return;
    }

    // 默认层数
    if (layers === undefined || layers === null) {
        layers = -1;
    }
    // 参数交换
    else if (typeof (layers) !== 'number') {
        doFun = layers;
        layers = -1;
    }

    var exclude: any; // 排除列表
    var exFun: any; // 排除处理函数
    if (doFun) {
        if (typeof (doFun) === 'function') {
            exFun = doFun;
        }
        else {
            exclude = doFun;
        }
    }

    for (var a in obj) {
        // 跳过数组排除
        if (exclude && exclude.indexOf(a) >= 0) {
            continue;
        }
        // 跳过函数排除
        else if (exFun && exFun(a, obj, dst) === true) {
            continue;
        }

        // 复制
        if ((layers !== 0) && Object.prototype.toString.call(obj[a]) === "[object Object]") {
            if (Object.prototype.toString.call(dst[a]) !== "[object Object]") {
                dst[a] = {};
            }
            unionObj(dst[a], obj[a], layers - 1, doFun); //递归遍历
        }
        else {
            dst[a] = obj[a];
        }
    }
}

/**
 * 获取随机整数
 * @param min 最小值
 * @param max 最大值
 * @returns 随机数
 */
export function getRndInt(min: number, max: number) {
    return Math.floor((Math.random() * (max - min + 1)) + min);
}

/**
 * 获取标记戳
 * @returns 标记戳字符串
 */
export function getJstamp() {
    return Date.now() + "_" + getRndInt(0, 9999999);
}

/**
 * 是否为yield可使用的函数
 * @param fun 函数
 * @returns 布尔值
 */
export { isYieldFun };

/**
 * 对比对象
 * @param dst 对象1
 * @param obj 对象2
 * @param exclude 排除的内容
 * @returns 无
 */
export function equalObj(dst: any, obj: any, exclude?: string[]) {
    if (dst === obj) return true;
    if (!dst || !obj) return false;
    for (var a in obj) {
        // 跳过排除
        if (exclude && exclude.indexOf(a) >= 0) continue;

        var objCurType = Object.prototype.toString.call(obj[a]);
        if (objCurType === "[object Object]" || objCurType === "[object Array]") {
            if (Object.prototype.toString.call(dst[a]) !== objCurType) {
                return false;
            }
            // 递归遍历
            if (equalObj(dst[a], obj[a], exclude) === false) {
                return false;
            }
        }
        else if (dst[a] !== obj[a]) {
            return false;
        }
    }

    return true;
}



export type ParamMultiDoCb = (err: Error | null, res: any, argInfo: ParamMultiDoTask, taskId: number) => void;
export type ParamMultiDoTask = {

    /**
     * 执行函数
     */
    fun: Function,

    /**
     * 执行函数参数数组
     */
    args: Array<any>,

    /**
     * this对象
     */
    thisObj?: any,

    /**
     * 执行回调
     */
    cbFun?: ParamMultiDoCb

    /**
     * 返回的结果
     */
    retRes?: any,

    /**
     * 返回的错误
     */
    retErr?: Error,

    /**
     * 用户对象
     */
    [key: string]: any

};

/**
 * 多任务并发处理
 * @param taskList 任务列表
 * @param startCount 并发数,省略后为全并发
 * @param errCount 发生指定错误数后终止任务
 * @param doCb 操作回调
 * @returns 执行的数量
 */
export function multiDo(taskList: Array<ParamMultiDoTask>, startCount?: number, errCount?: number, doCb?: ParamMultiDoCb) {

    // 等待列表
    var mainWaitList = [];

    // 当前位置
    var curIndex = 0;

    // 并发数
    if (!startCount) startCount = taskList.length;

    // 错误退出数量
    var errExit = errCount ? errCount : 999999999;

    // 并发任务
    for (var index = 0; index < startCount; index++) {

        // 建立任务
        mainWaitList.push(jco(function* (): any {

            // 任务ID
            var taskId = index;

            // 循环
            while (true) {
                // 判断数量
                if (errExit <= 0 || curIndex >= taskList.length) return null;

                // 取出一个
                var curTask = taskList[curIndex];
                var curNum = curIndex;
                curIndex++; // 下一个

                try {
                    // 执行
                    curTask.retRes = yield curTask.fun.apply(curTask.thisObj ? curTask.thisObj : null, curTask.args);

                    //成功任务回调
                    if (curTask.cbFun) {
                        try {
                            if (isYieldFun(curTask.cbFun)) {
                                yield curTask.cbFun(null, curTask.retRes, curTask, taskId);
                            }
                            else {
                                curTask.cbFun(null, curTask.retRes, curTask, taskId);
                            }
                        } catch (error) {
                            if (jlog.jlogEnable) jlog.error("multiDo[" + curNum + "] ", error);
                        }
                    }

                    //成功总回调
                    if (doCb) {
                        try {
                            if (isYieldFun(doCb)) {
                                yield doCb(null, curTask.retRes, curTask, taskId);
                            }
                            else {
                                doCb(null, curTask.retRes, curTask, taskId);
                            }
                        } catch (error) {
                            if (jlog.jlogEnable) jlog.error("multiDo[" + curNum + "] ", error);
                        }
                    }

                } catch (error: any) {
                    // 错误
                    errExit--;
                    curTask.retErr = error;
                    if (jlog.jlogEnable) jlog.error("multiDo[" + curNum + "] ", error);

                    // 错误任务回调
                    if (curTask.cbFun) {
                        try {
                            if (isYieldFun(curTask.cbFun)) {
                                yield curTask.cbFun(error, null, curTask, taskId);
                            }
                            else {
                                curTask.cbFun(error, null, curTask, taskId);
                            }
                        } catch (error) {
                            if (jlog.jlogEnable) jlog.error("multiDo[" + curNum + "] ", error);
                        }
                    }

                    // 错误总回调
                    if (doCb) {
                        try {
                            if (isYieldFun(doCb)) {
                                yield doCb(error, null, curTask, taskId);
                            }
                            else {
                                doCb(error, null, curTask, taskId);
                            }
                        } catch (error) {
                            if (jlog.jlogEnable) jlog.error("multiDo[" + curNum + "] ", error);
                        }
                    }

                }
            }

        }));

    }

    // 返回
    return Promise.all(mainWaitList).then(() => { return curIndex });
}



// 初始化整数ID
var intID = Date.now() - (new Date("2000-01-01 00:00:00")).getTime();
if (intID < 0) intID = -intID;
var strId = intID + "";
strId = strId.substr(0, 2) + strId.substr(-5) + strId.substr(2, strId.length - 7);
intID = parseInt(strId, 10);
// 留出自增段,js安全数最大为:2的53次方=9007199254740992
intID *= 1000

/**
 * 获取Int8的序列ID
 * @returns Int8的序列ID
 */
export function getId() {
    intID++;
    return intID;
}


/**
 * 获取当前时间字符串
 * @param mode 模式,不填=2,2="YYYY-MM-DD HH:mm:ss",3=IOS格式,"字符串自定义时间"=自定义(需要安装moment)
 * @returns 当前时间
 */
export function getNowDate(mode?: number | string) {
    return <string>jtype_utils.getDate(new Date(), undefined, mode ? mode : 2);
}

/**
 * double转日期型,以1900年为基础
 * @param doubleVal 
 * @returns Date对象
 */
export function doubleToDate(doubleVal: number) {
    var oDate = new Date();
    oDate.setTime((doubleVal - 25569) * 24 * 3600 * 1000 + (oDate.getTimezoneOffset() * 60000));
    return oDate;
}


/**
 * 获取当前时区小时,如: 8  -4
 */
export function getTimeZoneInt() {
    return Math.floor((-(new Date()).getTimezoneOffset()) / 60);
}

/**
 * 获取时区 如:+00:00 +08:00 -04:00
 * @param isNegative 使用负值,比如+08:00 -> -08:00
 */
export function getTimeZoneStr(isNegative?: boolean) {
    var timezoneInt = isNegative ? -getTimeZoneInt() : getTimeZoneInt();
    var timezoneStr = Math.abs(timezoneInt) + "";
    if (timezoneStr.length === 1) timezoneStr = "0" + timezoneStr;
    return ((timezoneInt >= 0) ? "+" : "-") + timezoneStr + ":00";
}

/**
 * 过滤SQL,防止注入
 * @param sqlParam sql参数
 */
export function filterSQL(sqlParam: string) {
    return sqlParam.replace(/(--|'|"|`|>|<|\[|\]|\\|\/|\{|\})/g, "_");
}

// 用于检测空对象
var chkInjectionEmpty = <any>{};

/**
 * 对象注入检查,有问题将会抛出错误
 * @param obj 检查对象
 * @param noErr 不抛出异常
 * @return 注入对象名称,如果为null表示正常
 */
export function chkInjection(obj: any, noErr?: boolean): null | string {
    if (!obj) return null;
    if (typeof (obj) === 'string') {
        if (chkInjectionEmpty[obj]) {
            if (noErr) return "v=" + obj;
            throw new Error("Injection Error,v=" + obj);
        }
    }
    else {
        for (var a in obj) {
            var ObjType = Object.prototype.toString.call(obj[a]);
            // 检测注入
            if (ObjType === "[object String]" && chkInjectionEmpty[obj[a]]) {
                if (noErr) return "k=" + a + ",v=" + obj[a];
                throw new Error("Injection Error,k=" + a + ",v=" + obj[a]);
            }
            else if (ObjType === "[object Object]" || ObjType === "[object Array]") {
                var ret = chkInjection(obj[a], noErr); //递归遍历
                if (ret) return ret
            }
        }
    }
    return null;
}


/**
 * 驼峰转下划线名
 * @param hStr 驼峰名称
 */
export function h2u(hStr: string) {
    var outStr = hStr[0];
    var charCode;
    for (var i = 1; i < hStr.length; i++) {
        charCode = hStr.charCodeAt(i);
        if (charCode >= 65 && charCode <= 90) {
            outStr += "_" + String.fromCharCode(charCode + 32)
        }
        else {
            outStr += hStr[i];
        }
    }
    return outStr;
}


/**
 * 下划线转驼峰名称
 * @param uStr 下划线名称
 */
export function u2h(uStr: string) {
    var i = 1;
    var outStr = uStr[0].toLowerCase();
    for (; i < uStr.length; i++) {
        if (uStr.charCodeAt(i) === 95) {
            outStr += String.fromCharCode(uStr.charCodeAt(i + 1) - 32);
            i++;
        }
        else {
            outStr += uStr[i];
        }
    }
    return outStr;
}


/**
 * 对比两个对象值
 * @param obj 
 */
function valueOfEx(obj: any) {
    return obj.valueOf ? obj.valueOf() : Object.prototype.valueOf.call(obj);
}

/**
 * 对比两个对象
 * @param a 对象A
 * @param b 对象B
 */
export function valueEqual(a: any, b: any): boolean {
    // Test for strict equality first.
    if (a === b) return true;

    // Otherwise, if either of them == null they are not equal.
    if (a == null || b == null) return false;

    if (Array.isArray(a)) {
        return (
            Array.isArray(b) &&
            a.length === b.length &&
            a.every(function (item, index) {
                return valueEqual(item, b[index]);
            })
        );
    }

    if (typeof a === 'object' || typeof b === 'object') {
        var aValue = valueOfEx(a);
        var bValue = valueOfEx(b);

        if (aValue !== a || bValue !== b) return valueEqual(aValue, bValue);

        return Object.keys(Object.assign({}, a, b)).every(function (key) {
            return valueEqual(a[key], b[key]);
        });
    }

    return false;
}


/**
 * 是否是空对象
 * @param obj 
 */
export function isEmptyObj(obj: any) {
    for (var key in obj) {
        return false;
    }
    return true;
}


