"use strict";
module.exports = function jco(gen) {
    // 执行Promise
    return new Promise(function (resolve, reject) {
        // GeneratorFun转Generator
        if (typeof (gen) === 'function') {
            try {
                gen = gen();
            }
            catch (error) {
                return reject(error);
            }
        }
        // 不是Generator直接返回结果
        if (!gen || typeof (gen.next) !== 'function')
            return resolve(gen);
        // 执行驱动
        onFulfilled();
        // 驱动处理
        function onFulfilled(res) {
            var ret;
            try {
                ret = gen.next(res);
            }
            catch (e) {
                return reject(e);
            }
            next(ret);
        }
        // 异常处理
        function onRejected(err) {
            var ret;
            try {
                ret = gen.throw(err); // 异常是否被处理
            }
            catch (e) {
                return reject(e); // 被继续抛出
            }
            next(ret); // 已处理,继续
        }
        // 下一步
        function next(ret) {
            // 结束判断
            if (ret.done)
                return resolve(ret.value);
            // 最终对象(一定是Promise)
            var obj = ret.value;
            // 如果不是Promise,继续嵌套co
            if (!obj || 'function' !== typeof (obj.then)) {
                obj = jco(obj);
            }
            // 执行已决
            return obj.then(onFulfilled, onRejected);
        }
    });
};
