"use strict";
const path = require("path");
// 错误名称列表
var errNameList = {};
/**
 * 通用错误(node.js从[-10001]开始定义)
 */
var errObj = {
    /**
     * 无错误*
     */
    OK: 0,
    /**
     * 未定义错误*
     */
    UNDEF: -1,
    /**
     * 超时*
     */
    TIME_OUT: -101,
    /**
     * 权限不足
     */
    ACCESS_DENIED: -102,
    /**
     * 找不到网页
     */
    HTTP_NOT_FOUND: -404,
    /**
     * 服务器错误
     */
    HTTP_SERVER_ERROR: -500,
    /**
     * 不支持的请求
     */
    HTTP_NOT_IMPLEMENTED: -501,
    /**
     * 错误的返回格式*
     */
    RET_FORMAT_ERR: -10001,
    /**
     * 参数丢失
     */
    PARAM_LOST: -10002,
    /**
     * JSON解析失败
     */
    JSON_PARSE_ERR: -10003,
    /**
     * 类型检查错误
     */
    CHK_TYPE_ERR: -10004,
    /**
     * RestApi处理错误
     */
    REST_RET_ERR: -10005,
    /**
     * 找不到指令
     */
    NO_FIND_CMD: -10006,
    /**
     * 参数错误
     */
    PARAM_ERROR: -10007,
    /**
     * 网络连接错误
     */
    HTTP_NET_ERR: -10008,
    /**
     * 网络服务器处理错误
     */
    HTTP_SER_ERR: -10009,
    /**
     * 文件系统错误
     */
    FILE_ERR: -10010,
    /**
     * 找不到服务工具
     */
    NO_SERVER_UTILS: -10011,
    /**
     * 对象缺失
     */
    OBJ_LOST_ERR: -10012,
    /**
     * 对象配置错误
     */
    OBJ_CFG_ERR: -10013,
    /**
     * 服务器错误
     */
    SER_ERR: -10014,
    /**
     * 第三方服务错误
     */
    EXT_SER_ERR: -10015,
    /**
     * 没有配置文件
     */
    NO_CONFIG_FILE: -10016,
    /**
     * 没有配置的参数
     */
    NO_CONFIG_PARAM: -10017,
    /**
     * 没有找到列表
     */
    NO_FIND_ITEM: -10018,
    /**
     * 用户取消
     */
    USER_ABOUT: -10019,
    // 导出功能函数
    getCurLine: getCurLine,
    getName: getName,
    errNameList: errNameList,
    setUserErr: setUserErr,
    setIdErr: setIdErr,
    newError: newError
};
/**
 * 获取错误名称
 * @param errno 错误码
 * @returns 错误名称
 */
function getName(errno) {
    return errNameList[errno] || (errno + "");
}
/**
 * 制作错误信息
 * @returns 无
 */
function makeErrInfo() {
    // 遍历错误名称
    for (var errName in errObj) {
        if (typeof (errObj[errName]) === "number") {
            errNameList[errObj[errName]] = errName;
        }
    }
}
/**
 * 获取日志行号
 * @param jumpFilename 跳过的文件名,比如"jlog.js"表示寻找这个文件之上的文件错误行
 * @param onlyNum 只包含行号
 * @returns 行号,小于0表示获取失败
 */
function getCurLine(jumpFilename, onlyNum) {
    // 取出堆栈信息
    var x1 = {};
    Error.captureStackTrace(x1);
    if (!x1.stack)
        return -1;
    try {
        // 跳过的文件名
        if (!jumpFilename)
            jumpFilename = "jerror_type.js";
        var chkStr = path.sep + jumpFilename + ":";
        // 搜索错误行信息
        var getSp = x1.stack.split("\n");
        for (var i = 0; i < getSp.length - 1; i++) {
            if (getSp[i].indexOf(chkStr) > 0) {
                // 下一行为代码错误行
                if (!onlyNum) {
                    return getSp[i + 1].trim();
                }
                var p1 = getSp[i + 1].indexOf(".js:");
                var LineNum = parseInt(getSp[i + 1].substr(p1 + 4, 15), 10);
                if (!isNaN(LineNum)) {
                    if (LineNum > 0)
                        return LineNum;
                }
            }
        }
    }
    catch (error) {
    }
    // 没找到行号
    return onlyNum ? -1 : "";
}
/**
 * 设置用户错误集
 * @param userErr 用户错误集
 * @returns 错误名称
 */
function setUserErr(userErr) {
    // 迁移属性
    for (var errName in userErr) {
        if (typeof (userErr[errName]) === "number") {
            errObj[errName] = userErr[errName];
        }
    }
    // 刷新信息
    makeErrInfo();
}
/**
 * 设置ID错误集
 * @param idErr ID错误集,{"-10":"pwd is xxxx"}
 * @returns 错误名称
 */
function setIdErr(idErr) {
    // 放入ID错误属性
    for (var errId in idErr) {
        errNameList[errId + ""] = idErr[errId];
    }
}
/**
 * 生成错误
 * @param errno 错误码
 * @param message 错误描述信息(可省)
 * @returns 错误名称
 */
function newError(errno, msg) {
    var errName = getName(errno);
    var message = (typeof (msg) === "string") ? msg : errName;
    var retErr = new Error(message);
    retErr.code = errName;
    retErr.errno = errno;
    if (msg instanceof Error)
        retErr.err = msg;
    return retErr;
}
// 载入时执行一次
makeErrInfo();
module.exports = errObj;
