"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.syncLanguageFile = syncLanguageFile;
exports.def = def;
const jtype_utils = require("./jtype_utils");
const json_utils = require("./json_utils");
const jbase_utils = require("./jbase_utils");
const fs = require("fs");
const Jlog = require("./jlog");
var jlog = Jlog("jlng");
/**
 * 语言对象
 */
var lngObj = {};
/**
 * 同步文件选项
 */
var fileSyncOpt = {
    fileName: "",
    defLng: "",
    userLng: "",
    defPriority: false
};
/**
 * 同步标记,true=需要同步
 */
var syncSig = false;
/**
 * 文件对象
 */
var fileObj = {};
/**
 * 同步次数
 */
var syncCount = 0;
/**
 * 公共字符串
 */
var pubStrList = {};
/**
 * 同步语言文件
 * @param lngFileName 文件路径或路径数组
 * @param defLng 定义语言(在写代码时候的语言)
 * @param userLng 用户设置的语言(用户配置文件指定要看的语言)
 * @param defPriority 定义语言优先(在写代码时候的语言)
 */
function syncLanguageFile(lngFileName, defLng, userLng, defPriority) {
    if (!defLng)
        defLng = "cn";
    if (!userLng)
        userLng = "cn";
    // 支持数组文件
    var lngFileList = jtype_utils.getArray(lngFileName, [lngFileName]);
    var fileName = lngFileList[0];
    // 载入多语言文件
    if (fileName) {
        // 保存路径
        fileSyncOpt.fileName = fileName;
        fileSyncOpt.defLng = defLng;
        fileSyncOpt.userLng = userLng;
        fileSyncOpt.defPriority = defPriority;
        for (var index = 0; index < lngFileList.length; index++) {
            try {
                var curObj = JSON.parse(json_utils.jsonTrim(fs.readFileSync(lngFileList[index], "utf8")));
                jbase_utils.unionObj(fileObj, curObj);
                if (jlog.jlogEnable)
                    jlog.info("load file:" + lngFileList[index]);
            }
            catch (error) {
                if (jlog.jlogEnable)
                    jlog.info("no load:" + lngFileList[index]);
            }
        }
    }
    else {
        if (jlog.jlogEnable)
            jlog.info("refresh");
    }
    var needWrite = false;
    // 同步次数
    syncCount++;
    var subObj2;
    var lngDef2;
    // 生产多语言序列
    for (const key1 in lngObj) {
        const lngDef1 = lngObj[key1];
        // 标号
        var newLngList = {};
        for (const key2 in lngDef1.lngList) {
            lngDef2 = lngDef1.lngList[key2];
            // 补文件段
            if (!fileObj[lngDef1.lngName])
                fileObj[lngDef1.lngName] = {};
            var subObj1 = fileObj[lngDef1.lngName];
            // 补全标号值
            if (!subObj1[key2]) {
                subObj1[key2] = {};
            }
            subObj2 = subObj1[key2];
            // 优先定义语言
            if (subObj2[defLng] !== lngDef2 && defPriority) {
                subObj2[defLng] = lngDef2;
            }
            // 补充空缺
            if (!subObj2[defLng]) {
                subObj2[defLng] = lngDef2;
            }
            else if (!defPriority) {
                // 优先外部文件
                lngDef2 = subObj2[defLng];
            }
            // 存在用户语言
            if (subObj2[userLng]) {
                // 载入
                newLngList[key2] = subObj2[userLng];
            }
            else if (subObj2[userLng] === undefined) {
                // 配置为空白
                subObj2[userLng] = "";
                // 使用文档语言
                newLngList[key2] = lngDef2;
                needWrite = true;
            }
            else {
                // 使用文档语言
                newLngList[key2] = lngDef2;
            }
        }
        // 替换掉旧数据
        lngDef1.lngList = newLngList;
    }
    // 公共语言
    if (!fileObj.public) {
        fileObj.public = {
            DEMO: {
                cn: "这是一个demo,元素最好大写"
            }
        };
        // 回写
        needWrite = true;
    }
    ;
    // 导入公共语言
    for (const key in fileObj.public) {
        const element = fileObj.public[key];
        pubStrList[key.toUpperCase()] = element.cn;
    }
    // 写出
    if (needWrite && fileSyncOpt.fileName) {
        fs.writeFileSync(fileSyncOpt.fileName, JSON.stringify(fileObj, null, 4));
        if (jlog.jlogEnable)
            jlog.info("write file:" + fileSyncOpt.fileName);
    }
}
/**
 * 定义并返回多语言函数
 * @param jsfile 脚本文件
 */
function def(jsfile) {
    // 创建导出函数
    var lngFun = {
        lngName: "default",
        lngList: {}
    };
    // 导入文件名
    if (typeof (jsfile) === "string") {
        if (process.cwd().length > jsfile.length) {
            lngFun.lngName = jsfile; // 自定义名称
        }
        else {
            lngFun.lngName = jsfile.substr(process.cwd().length + 1); // 使用文件名
        }
    }
    else {
        throw new Error("no lng file name!");
    }
    // 对象
    lngObj[lngFun.lngName] = lngFun;
    // 返回函数
    return function (jindex, text, text1, text2, text3, text4, text5, text6, text7, text8, text9) {
        var txtIndex = (jindex + "").toUpperCase();
        // 查找文本字符串
        var txtFmt = pubStrList[txtIndex];
        if (!txtFmt) {
            txtFmt = lngFun.lngList[txtIndex];
        }
        // 有填充文本
        if (text !== undefined) {
            text = text + "";
            // 找到填充Fmt,格式化输出
            if (txtFmt) {
                // 格式化
                var spTxt = txtFmt.split("%s");
                var retStr = spTxt[0];
                var index = 1;
                for (; index < spTxt.length; index++) {
                    retStr += arguments[index] + spTxt[index];
                }
                for (; index < arguments.length; index++) {
                    retStr += " " + arguments[index];
                }
                return retStr;
            }
            else {
                // 没找到,视为赋值
                lngFun.lngList[txtIndex] = text.replace(/\%d/g, "%s");
                // 重加载
                if (syncCount) {
                    if (!syncSig) {
                        syncSig = true;
                        if (jlog.jlogEnable)
                            jlog.warn(lngFun.lngName + " old");
                        setTimeout(() => {
                            if (syncSig) {
                                syncSig = false;
                                syncLanguageFile(null, fileSyncOpt.defLng, fileSyncOpt.userLng, fileSyncOpt.defPriority);
                            }
                        }, 0);
                    }
                }
            }
        }
        return (typeof (txtFmt) === "string") ? txtFmt : "undefined";
    };
}
;
