"use strict";
/**
 * 调试开关
 */
var useDebug = (process.env.DEBUG || process.env.JDEBUG) ? true : false;
// 无颜色输出
var noColorOut = !!process.env.JCTL_NOCOLOR; //!!process.env.VSCODE_INSPECTOR_OPTIONS;
// 默认配置
var defCfg = {
    hideInit: true, // 隐藏初始化
    showLineSw: "000011", // 行号显示开关
    levelSw: "111111" // 日志记录等级开关
};
// 变量
var extOutObj = {}; // 扩展输出
var showLineSw = null; // 行号显示开关
var levelSw = null; // 日志等级开关
var logObj = {}; // 日志对象
/**
 * 子元素最大长度
 */
var logSubLength = 512;
/**
 * 记录最大长度
 */
var logAllLength = 1024;
/**
 * 初始化,若没有变动无需初始化
 * @param cfg 配置
 * {levelSw:等级输出开关,默认"111111",showLineSw:行号开关,默认"000011"}
 * 开关项由6个字符的字符串表示,如:"001111",分别表示trace,debug,info,warn,error,fatal
 * @returns 无
 */
function config(cfg) {
    // 开始配置
    var showLineSw_cur = defCfg.showLineSw;
    var levelSw_cur = defCfg.levelSw;
    if (cfg) {
        // 行号开关存储
        if (cfg.showLineSw && cfg.showLineSw.length >= 6) {
            showLineSw_cur = cfg.showLineSw;
        }
        // 日志等级开关存储
        if (cfg.levelSw && cfg.levelSw.length >= 6) {
            levelSw_cur = cfg.levelSw;
        }
    }
    // 入参转换
    if (Buffer.alloc) {
        showLineSw = Buffer.from(showLineSw_cur);
        levelSw = Buffer.from(levelSw_cur);
    }
    else {
        showLineSw = new Buffer(showLineSw_cur);
        levelSw = new Buffer(levelSw_cur);
    }
    // 输出
    if (cfg && !(cfg.hideInit)) {
        logOutLine("jlog.js", 2, ["jlog init ok,levelSw:\"" + cfg.levelSw + "\""]);
    }
}
// 默认初始化
config(defCfg);
/**
 * 日志写出
 * @param jsName js文件名
 * @param logLevel 日志等级
 * @param logStr 日志字符串
 */
function consoleWrite(logObj) {
    var tim1 = new Date();
    var logHeadStr;
    var logLevelStr;
    switch (logObj.logLevel) {
        case 0:
            logHeadStr = "\x1B[90m[";
            logLevelStr = "t";
            break;
        case 1:
            logHeadStr = "\x1B[32m[";
            logLevelStr = "d";
            break;
        case 2:
            logHeadStr = "\x1B[36m[";
            logLevelStr = "i";
            break;
        case 3:
            logHeadStr = "\x1B[33m[";
            logLevelStr = "w";
            break;
        case 4:
            logHeadStr = "\x1B[31m[";
            logLevelStr = "e";
            break;
        case 5:
            logHeadStr = "\x1B[35m[";
            logLevelStr = "f";
            break;
        default:
            logHeadStr = "\x1B[31m[";
            logLevelStr = "unknow";
            break;
    }
    // 组织内容
    var retStr = `[${logLevelStr}]${logHeadStr}${("00" + tim1.getDate()).substr(-2)} ${("00"
        + tim1.getHours()).substr(-2)}:${("00" + tim1.getMinutes()).substr(-2)}:${("00"
        + tim1.getSeconds()).substr(-2)}.${("000" + tim1.getMilliseconds()).substr(-3)}][${logObj.jsName}]\x1B[39m:${logObj.logStr}`;
    // IDE输出
    if (noColorOut) {
        var retStr = `[${logLevelStr}]${("00" + tim1.getDate()).substr(-2)} ${("00"
            + tim1.getHours()).substr(-2)}:${("00" + tim1.getMinutes()).substr(-2)}:${("00"
            + tim1.getSeconds()).substr(-2)}.${("000" + tim1.getMilliseconds()).substr(-3)}][${logObj.jsName}]:${logObj.logStr}`;
        if (logObj.logLevel > 3) {
            console.error(retStr);
        }
        else if (logObj.logLevel > 2) {
            console.warn(retStr);
        }
        else {
            console.log(retStr);
        }
    }
    // 控制台输出
    else {
        var retStr = `[${logLevelStr}]${logHeadStr}${("00" + tim1.getDate()).substr(-2)} ${("00"
            + tim1.getHours()).substr(-2)}:${("00" + tim1.getMinutes()).substr(-2)}:${("00"
            + tim1.getSeconds()).substr(-2)}.${("000" + tim1.getMilliseconds()).substr(-3)}][${logObj.jsName}]\x1B[39m:${logObj.logStr}`;
        if (logObj.logLevel < 4) {
            console.log(retStr);
        }
        else {
            console.error(retStr);
        }
    }
    // // 控制台输出
    // if (logObj.logLevel < 4) {
    //     process.stdout.write(retStr);
    //     process.stdout.write("\n");
    // }
    // else {
    //     process.stderr.write(retStr);
    //     process.stderr.write("\n");
    // }
}
/**
 * 添加扩展输出函数
 * @param writeFun // 扩展写出函数
 * @param extName // 名称
 * @returns 无
 */
function addLogOut(writeFun, extName) {
    if (!extName)
        extName = "default";
    delete extOutObj[extName];
    extOutObj[extName] = {};
    extOutObj[extName].writeFun = writeFun;
}
/**
 * 添加日志函数类
 * @param logTarget 日志函数类,包含logTarget.debug,logTarget.info,logTarget.error....等
 * @param extName // 名称
 */
function addLogClass(logTarget, extName, firstParam) {
    // 日志写出
    addLogOut((jsName, logLevel, logStr) => {
        // 组织参数
        var logParam;
        if (firstParam) {
            logParam = [firstParam, "[" + jsName + "]:" + logStr];
        }
        else {
            logParam = ["[" + jsName + "]:" + logStr];
        }
        // 根据类型输出,这里是故意没有break的,这样可以让没有定义的输出有意义
        switch (logLevel) {
            case 0:
                if (logTarget.trace) {
                    logTarget.trace(...logParam);
                    return;
                }
                if (logTarget.TRACE) {
                    logTarget.TRACE(...logParam);
                    return;
                }
            case 1:
                if (logTarget.debug) {
                    logTarget.debug(...logParam);
                    return;
                }
                if (logTarget.DEBUG) {
                    logTarget.DEBUG(...logParam);
                    return;
                }
            case 2:
                if (logTarget.info) {
                    logTarget.info(...logParam);
                    return;
                }
                if (logTarget.INFO) {
                    logTarget.INFO(...logParam);
                    return;
                }
            case 3:
                if (logTarget.warn) {
                    logTarget.warn(...logParam);
                    return;
                }
                if (logTarget.WARN) {
                    logTarget.WARN(...logParam);
                    return;
                }
            case 5:
                if (logTarget.fatal) {
                    logTarget.fatal(...logParam);
                    return;
                }
                if (logTarget.FATAL) {
                    logTarget.FATAL(...logParam);
                    return;
                }
            case 4:
                if (logTarget.err) {
                    logTarget.err(...logParam);
                    return;
                }
                if (logTarget.error) {
                    logTarget.error(...logParam);
                    return;
                }
                if (logTarget.ERROR) {
                    logTarget.ERROR(...logParam);
                    return;
                }
            default:
                break;
        }
        // 未提供的函数
        console.error("noOutLog", logParam);
    }, extName);
}
/**
 * 删除扩展输出函数
 * @param extName // 扩展名,为空则删除所有
 * @returns 无
 */
function removeExOut(extName) {
    if (extName) {
        delete extOutObj[extName];
    }
    else {
        for (var curExt in extOutObj) {
            delete extOutObj[curExt];
        }
    }
}
/**
 * 获取日志行号
 * @returns 无
 */
function getLogLine() {
    // 取出行号
    var x1 = {};
    Error.captureStackTrace(x1);
    if (!x1.stack)
        return -1;
    try {
        var lastLine = 0;
        // 找到最后一个log文件的日志位置
        var lineIndex = 0;
        do {
            lineIndex = x1.stack.indexOf("jlog.js:", lastLine);
            if (lineIndex > lastLine)
                lastLine = lineIndex + 6;
        } while (lineIndex >= 0);
        // 取出行号
        if (lastLine) {
            var callLine = x1.stack.indexOf(".js:", lastLine + 6);
            var LineNum = parseInt(x1.stack.substr(callLine + 4, 15), 10);
            if (!isNaN(LineNum)) {
                if (LineNum > 0)
                    return LineNum;
            }
        }
    }
    catch (error) {
        return -2;
    }
    return -3;
}
/**
 * 反序列化过滤器
 * @param key
 * @param value
 */
function logReplacer(key, value) {
    var objType = Object.prototype.toString.call(value);
    switch (objType) {
        case "[object String]":
            {
                if (value.length > logSubLength) {
                    return value.substr(0, logSubLength) + "...";
                }
                break;
            }
        case "[object Array]":
        case "[object Uint8Array]":
            {
                if (value.length > 256) {
                    return "[" + Object.prototype.toString.call(value[0]) + "*" + value.length + "]";
                }
                break;
            }
        default:
    }
    return value;
}
/**
 * 日志写出函数
 * @param jsName 名称
 * @param logLevel 日志等级
 * @param argArr 参数数组
 * @returns 无
 */
function logOutLine(jsName, logLevel, argArr) {
    // 等级过滤
    if (levelSw[logLevel] !== 49)
        return;
    // 多变量
    var logStr = "";
    var errObj = null;
    for (var i = 0; i < argArr.length; i++) {
        var curObj = argArr[i];
        switch (typeof (curObj)) {
            case "object": // 对象
                try {
                    // 错误对象
                    if ((curObj.stack !== undefined && curObj.message !== undefined)
                        || Object.prototype.toString.call(curObj) === '[object Error]') {
                        errObj = curObj;
                        logStr += "[object Error]\n";
                        var pNameList = Object.getOwnPropertyNames(curObj);
                        for (var j = 0; j < pNameList.length; j++) {
                            var prName = pNameList[j];
                            if (Object.prototype.toString.call(curObj[prName]) === '[object Error]') {
                                var subObj = curObj[prName];
                                var subNameList = Object.getOwnPropertyNames(subObj);
                                for (var k = 0; k < subNameList.length; k++) {
                                    var subName = subNameList[k];
                                    logStr += prName + ":" + subName + ":" + subObj[subName] + "\n";
                                }
                            }
                            else {
                                logStr += prName + ":" + curObj[prName] + "\n";
                            }
                        }
                    }
                    else {
                        // 标准对象
                        logStr += JSON.stringify(curObj, logReplacer);
                    }
                }
                catch (error) {
                    logStr += "[jlog error]:error object!";
                }
                break;
            case "function": // 函数
                logStr += "<" + curObj.name + ">";
                break;
            default: // 其他
                logStr += curObj;
                break;
        }
        // 分隔
        logStr += " ";
    }
    //裁剪
    if (logLevel < 4 && logStr.length > logAllLength)
        logStr = logStr.substr(0, logAllLength) + "...";
    // 输出
    if (showLineSw[logLevel] === 49) {
        jsName += ":" + getLogLine(); // 行号
    }
    var ret = 0;
    var curExt;
    for (curExt in extOutObj) {
        ret += extOutObj[curExt].writeFun(jsName, logLevel, logStr) ? 1 : 0;
    }
    // 没有输出工具或者需要输出
    if (!curExt || ret > 0) {
        consoleWrite({ jsName, logLevel, logStr, errObj });
    }
}
module.exports = function (jsfile) {
    // 创建导出函数
    var logFun = {
        /**
         * JS文件名
         */
        jsName: "",
        /**
         * 日志自身启用开关
         */
        jlogEnable: true,
        /**
         * 初始化,若没有变动无需初始化
         * @param cfg 配置
         * {levelSw:等级输出开关,默认"111111",showLineSw:行号开关,默认"000011"}
         * 开关项由6个字符的字符串表示,如:"001111",分别表示trace,debug,info,warn,error,fatal
         * @returns 无
         */
        config: config,
        /**
         * 添加日志函数类
         * @param logTarget 日志函数类,包含logTarget.debug,logTarget.info,logTarget.error....等
         * @param extName // 名称
         */
        addLogClass: addLogClass,
        /**
         * 日志等级开关选项
         * @param newSwitch // 新的等级开关6个0或1的字符串,如:"001111"
         */
        setLevelSwitch: (newSwitch) => {
            levelSw.write(newSwitch, 0, 6);
        },
        /**
         * 日志行号开关选项
         * @param newSwitch // 新的行号等级开关6个0或1的字符串,如:"001111"
         */
        setShowLineSwitch: (newSwitch) => {
            showLineSw.write(newSwitch, 0, 6);
        },
        /**
         * 日志高级选项
         * @param opt // 新的行号等级开关6个0或1的字符串,如:"001111"
         */
        setAdvOpt: (opt) => {
            if (opt.subLength !== undefined)
                logSubLength = opt.subLength;
            if (opt.allLength !== undefined)
                logAllLength = opt.allLength;
        },
        /**
         * 单文件日志开关选项
         * @param isEnable 是否启用
         * @param logName 日志名,省略后为当前日志
         */
        setEnable: (isEnable, logName) => {
            if (logName) {
                if (logName === "*") {
                    if (useDebug)
                        console.error("[enable]:AllEnable=" + isEnable);
                    // 所有
                    for (var key in logObj) {
                        if (logObj[key]) {
                            logObj[key].jlogEnable = isEnable;
                        }
                    }
                }
                else if (logObj[logName]) {
                    // 指定
                    logObj[logName].jlogEnable = isEnable;
                }
            }
            else {
                logFun.jlogEnable = isEnable;
            }
            return logFun;
        },
        /**
         * 设置为环境开关
         * @param defEnable 默认启用状态
         */
        setSwFromEnv: (defEnable) => {
            // 随环境变量
            logFun.jlogEnable = defEnable ? true : false;
            // 环境变量匹配
            if (typeof (process.env.JDEBUG) === "string") {
                var debugArry = process.env.JDEBUG.split(",");
                if (process.env.JDEBUG === "*" || debugArry.indexOf(logFun.jsName) >= 0) {
                    logFun.jlogEnable = true;
                }
            }
            return logFun;
        },
        /**
         * 开发输出
         */
        trace: (function () {
            if (logFun.jlogEnable)
                logOutLine(logFun.jsName, 0, arguments);
        }),
        /**
         * 调试输出
         */
        debug: (function () {
            if (logFun.jlogEnable)
                logOutLine(logFun.jsName, 1, arguments);
        }),
        /**
         * 信息输出
         */
        info: (function () {
            if (logFun.jlogEnable)
                logOutLine(logFun.jsName, 2, arguments);
        }),
        /**
         * 警告输出
         */
        warn: (function () {
            if (logFun.jlogEnable)
                logOutLine(logFun.jsName, 3, arguments);
        }),
        /**
         * 错误输出
         */
        err: (function () {
            if (logFun.jlogEnable)
                logOutLine(logFun.jsName, 4, arguments);
            return arguments[0];
        }),
        /**
         * 错误输出
         * @returns 第一个参数
         */
        error: (function () {
            if (logFun.jlogEnable)
                logOutLine(logFun.jsName, 4, arguments);
            return arguments[0];
        }),
        /**
         * 错误输出+抛出新错误
         */
        errorThrow: (function () {
            if (logFun.jlogEnable)
                logOutLine(logFun.jsName, 4, arguments);
            throw new Error("errorThrow"); // 抛出异常
        }),
        /**
         * 故障输出
         * @returns 第一个参数
         */
        fatal: (function () {
            if (logFun.jlogEnable)
                logOutLine(logFun.jsName, 5, arguments);
            return arguments[0];
        }),
        /**
         * 故障输出+抛出新错误
         */
        fatalThrow: (function () {
            if (logFun.jlogEnable)
                logOutLine(logFun.jsName, 5, arguments);
            throw new Error("fatalThrow"); // 抛出异常
        }),
        /**
         * 添加扩展输出函数
         * @param writeFun // 扩展写出函数
         * @param extName // 名称(省略后为默认输出)
         * @returns 无
         */
        addExOut: (writeFun, extName) => {
            addLogOut(writeFun, extName);
            logOutLine("jlog.js", 2, ["addExOut:" + extName]);
        },
        /**
         * 删除扩展输出函数
         * @param extName // 扩展名,为空则删除所有
         * @returns 无
         */
        removeExOut: (extName) => {
            logOutLine("jlog.js", 2, ["delExOut:" + (extName ? extName : "All")]);
            removeExOut(extName);
        }
    };
    // 日志文件名
    logFun.jsName = "default";
    if (typeof (jsfile) === "string") {
        if (process.cwd().length > jsfile.length) {
            logFun.jsName = jsfile; // 自定义名称
        }
        else {
            logFun.jsName = jsfile.substr(process.cwd().length + 1); // 使用文件名
        }
        //if (useDebug) console.error("[start]:" + logFun.jsName);
    }
    else {
        //if (useDebug) console.error("[start]:no file name");
    }
    // 日志对象
    logObj[logFun.jsName] = logFun;
    // 返回
    return logFun;
};
