"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.syncLogFile = syncLogFile;
exports.cleanLogFile = cleanLogFile;
exports.toLog = toLog;
exports.showLog = showLog;
exports.logUtilsInit = logUtilsInit;
exports.getLogFilePath = getLogFilePath;
const fs = require("fs");
const path = require("path");
const jfile_utils = require("./jfile_utils");
const jbase_utils = require("./jbase_utils");
const jlog = require("./jlog");
const jco = require("./jco");
// ================ ↓配置项↓ ================
var m_logProcbtMs = 500; // 日志处理间隔(通常不需要设置)
var m_logSyncBtMs = 5000; // 日志强制同步磁盘间隔
var m_logFileMaxSize = 50 * 1024 * 1024; // 最大文件大小
var m_logKeepCount = 20; // 最多文件数量
var m_logFilePath = "./jlogs"; // 日志目录
var m_logPartName = ""; // 日志个性名
// ================ ↑配置项↑ ================
// 应用变量
var logfileSize = 0; // 日志文件大小
var logWriteList = []; // 日志待写入内容
var logfile = 0; // 日志文件句柄
var curLogPath = ""; // 日志路径
var logEnable = false; // 日志总开关
var disConsole = false; // 关闭控制台显示
var diskSyncLeft = 0; // 磁盘同步剩余,每500ms减1直到为0,到0开始同步一次
/**
 * 写入日志
 * @param logStr 日志内容
 */
function writeLog(logStr) {
    if (!logEnable)
        return;
    logWriteList.push(logStr);
}
/**
 * 日志文件同步磁盘
 * @returns {Promise<void>} 无
 */
function syncLogFile() {
    return jco(function* () {
        // 标记为立即同步
        diskSyncLeft = 1;
        // 等待同步完成
        // eslint-disable-next-line no-unmodified-loop-condition
        while (diskSyncLeft) {
            yield jbase_utils.sleep(100);
        }
    });
}
/**
 * 清理日志文件
 * @param keepCount 保留数量
 */
function cleanLogFile(keepCount) {
    return jco(function* () {
        if (!keepCount)
            keepCount = 5;
        try {
            var fileList = fs.readdirSync(m_logFilePath);
            fileList.sort(function (a, b) {
                return parseInt("0" + a, 10) - parseInt("0" + b, 10);
            });
            for (var index = 0; index < fileList.length - keepCount; index++) {
                var curFile = fileList[index];
                if (curFile[0] !== '.') {
                    jfile_utils.deleteFile(m_logFilePath + "/" + curFile);
                }
            }
        }
        catch (error) {
        }
    });
}
/**
 * 输出到日志
 * @param jsName 脚本名
 * @param logLevel 日志等级
 * @param logStr 日志字符串
 * @returns 是否继续输出到控制台
 */
function toLog(jsName, logLevel, logStr) {
    var tim1 = new Date();
    var logLevelStr;
    switch (logLevel) {
        case 0:
            logLevelStr = "t";
            break;
        case 1:
            logLevelStr = "d";
            break;
        case 2:
            logLevelStr = "i";
            break;
        case 3:
            logLevelStr = "w";
            break;
        case 4:
            logLevelStr = "e";
            break;
        case 5:
            logLevelStr = "f";
            break;
        default:
            logLevelStr = "unknow";
            break;
    }
    // 组织内容
    var retStr = `[${logLevelStr}][${("00" + tim1.getDate()).substr(-2)} ${("00"
        + tim1.getHours()).substr(-2)}:${("00" + tim1.getMinutes()).substr(-2)}:${("00"
        + tim1.getSeconds()).substr(-2)}.${("000" + tim1.getMilliseconds()).substr(-3)}][${jsName}]:${logStr}`;
    // 输出到文件
    writeLog(retStr);
    // 输出到控制台
    if (!disConsole) {
        // 继续输出控制台
        return true;
    }
    return false;
}
/**
 * 显示日志
 * @param opt 选项
 */
function showLog(opt) {
    disConsole = false;
}
/**
 * 创建新日志文件
 */
function createNewLogSync() {
    // 已经存在
    if (logfile) {
        return;
    }
    // 建立新文件
    var obj = new Date();
    curLogPath = m_logFilePath + "/" + obj.getFullYear() + ("00" + (obj.getMonth() + 1)).substr(-2) + ("00" + obj.getDate()).substr(-2)
        + ("00" + obj.getHours()).substr(-2) + ("00" + obj.getMinutes()).substr(-2) + ("00" + obj.getSeconds()).substr(-2)
        + (m_logPartName ? "_" + m_logPartName : "") + ".log";
    logfile = fs.openSync(curLogPath, "w");
    if (!logfile) {
        throw new Error("no file");
    }
}
/**
 * 强制写出所有日志数据,通常用于退出
 */
function forceWriteAllLog() {
    // 无数据跳出
    if (!logWriteList.length)
        return;
    // 新建文件
    createNewLogSync();
    // 写入
    var wStr = logWriteList.join("\r\n") + "\r\n";
    logWriteList = [];
    logfileSize += wStr.length;
    fs.writeSync(logfile, wStr);
    // 关闭日志文件
    fs.closeSync(logfile);
    logfile = 0;
    logfileSize = 0;
}
/**
 * 日志写到文件
 */
function* logToFile() {
    // 无数据跳出
    if (!logWriteList.length)
        return;
    try {
        // 打开文件
        if (!logfile) {
            // 清理
            yield cleanLogFile(m_logKeepCount);
            // 新建文件
            createNewLogSync();
        }
        // 写入
        var wStr = logWriteList.join("\r\n") + "\r\n";
        logWriteList = [];
        logfileSize += wStr.length;
        yield jfile_utils.write(logfile, wStr);
        // 需要同步(单位logProcbtMs)
        if (!diskSyncLeft) {
            diskSyncLeft = Math.floor(m_logSyncBtMs / m_logProcbtMs) + 1;
        }
        // 终止文件
        if (logfileSize > m_logFileMaxSize) {
            yield jfile_utils.close(logfile);
            logfile = 0;
            logfileSize = 0;
        }
    }
    catch (error) {
        // 发生无法处理的错误
        // eslint-disable-next-line no-console
        console.error(error);
        // eslint-disable-next-line no-console
        console.error(logWriteList);
        // 防止死循环,清空日志
        logWriteList = [];
        if (logfile) {
            var t = logfile;
            logfile = 0;
            try {
                yield jfile_utils.close(t);
            }
            catch (error2) {
            }
        }
    }
}
/**
 * 日志工具初始化(同步)
 * @param opt 选项
 */
function logUtilsInit(opt) {
    opt = opt || {};
    // 参数
    m_logProcbtMs = opt.logProcbtMs || m_logProcbtMs; // 日志写入间隔ms
    m_logSyncBtMs = opt.logSyncBtMs || m_logSyncBtMs; // 日志同步间隔s
    m_logFileMaxSize = opt.logFileMaxSize || m_logFileMaxSize; // 最大文件大小
    m_logKeepCount = opt.logKeepCount || m_logKeepCount; // 最多文件数量
    m_logPartName = opt.logPartName || m_logPartName; // 个性名称
    m_logFilePath = path.resolve(opt.logFilePath || m_logFilePath); // 日志目录
    // 创建日志目录
    jfile_utils.createDirs(m_logFilePath);
    // 总开关
    logEnable = true;
    // 是否输出到控制台
    disConsole = opt.disConsole || false;
    // 异步日志收集
    jco(function* () {
        while (true) {
            // 写日志到文件(不报错)
            yield logToFile();
            // 需要同步磁盘
            if (diskSyncLeft > 0) {
                diskSyncLeft--;
                if (diskSyncLeft === 0) {
                    try {
                        // 强制同步文件到磁盘
                        yield new Promise(function (resolve, reject) {
                            if (logfile) {
                                fs.fsync(logfile, resolve);
                            }
                            else {
                                resolve(true);
                            }
                        });
                    }
                    catch (error) {
                        // eslint-disable-next-line no-console
                        console.error(error);
                    }
                }
            }
            // 间隔写入
            yield jbase_utils.sleep(m_logProcbtMs);
        }
    });
    // nodeJS
    try {
        if (typeof (global) !== 'undefined') {
            jlog(__filename).addExOut(toLog, "wsLog");
            // 绑定全局log
            var globalLog = jlog("global");
            globalLog.showLog = showLog;
            global.jlog = globalLog;
            // 绑定退出函数
            process.on('exit', (code) => {
                try {
                    forceWriteAllLog();
                }
                catch (error) {
                    console.error(error);
                }
            });
        }
    }
    catch (error) {
    }
    // jvue
    setTimeout(function () {
        if (typeof ($jvue) !== 'undefined') {
            $jvue.init({
                logOut: function (logObj) {
                    toLog("jlog", logObj.logLevel, logObj.logStr);
                }
            });
        }
    }, 0);
}
/**
 * 获取日志文件路径
 * @return {string} 日志文件路径
 */
function getLogFilePath() {
    return curLogPath;
}
