"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFreePort = getFreePort;
exports.getIpAddr = getIpAddr;
exports.getMaskInt = getMaskInt;
const jtype_utils = require("./jtype_utils");
const jco = require("./jco");
const net = require("net");
/**
 * 获取空闲TCP端口
 * @param startPort 开始端口
 * @returns 空闲端口
 */
function getFreePort(startPort) {
    return jco(function* () {
        if (!startPort)
            startPort = 0;
        while (startPort < 65536) {
            try {
                // 检测内网绑定
                yield new Promise((resolve, reject) => {
                    var server = net.createServer(function () { });
                    server.on('error', (error) => {
                        reject(error);
                    });
                    server.listen(startPort, "127.0.0.1", function () {
                        var myAddr = server.address();
                        startPort = myAddr.port || 0;
                        server.close(() => {
                            resolve();
                        });
                    });
                });
                // 检测所有绑定
                yield new Promise((resolve, reject) => {
                    var server = net.createServer(function () { });
                    server.on('error', (error) => {
                        reject(error);
                    });
                    server.listen(startPort, "0.0.0.0", function () {
                        var myAddr = server.address();
                        startPort = myAddr.port || 0;
                        server.close(() => {
                            resolve();
                        });
                    });
                });
                return startPort;
            }
            catch (error) {
            }
            // 下一个
            startPort++;
        }
        return 0;
    });
}
/**
 * 获取正规的Ip地址
 * @param ipAddr ip地址
 * @param defAddr 解析错误后的ip地址
 * @returns Ip地址
 */
function getIpAddr(ipAddr, defAddr) {
    try {
        // 拆解
        var ipSp = ipAddr.split(".");
        if (!ipSp[3])
            return defAddr;
        // 转换数字
        var retIpSp = [0, 0, 0, 0];
        for (var index = 0; index < 4; index++) {
            retIpSp[index] = jtype_utils.getInt(parseInt(ipSp[index], 10), -1, 0, 255);
            if (retIpSp[index] < 0)
                return defAddr;
        }
        // 拼接
        return retIpSp.join(".");
    }
    catch (error) {
    }
    return defAddr;
}
/**
 * 获取子网掩码Int值
 * @param ipAddr ip地址
 * @param defInt 解析错误后的值
 * @returns Int值,如:8
 */
function getMaskInt(ipAddr, defInt) {
    if (!defInt)
        defInt = 0;
    try {
        // 拆解
        var ipSp = ipAddr.split(".");
        if (!ipSp[3])
            return defInt;
        // 转换数字
        var retIpSp = [0, 0, 0, 0];
        var i = 0;
        for (i = 0; i < 4; i++) {
            retIpSp[i] = jtype_utils.getInt(parseInt(ipSp[i], 10), -1, 0, 255);
            if (retIpSp[i] < 0)
                return defInt;
        }
        // 解析掩码剩余位
        var retIpBL = Buffer.from(retIpSp).readUInt32BE(0);
        var maskLeft = 0;
        for (i = 0; i < 32; i++) {
            if (retIpBL & 1)
                return (32 - i);
            retIpBL >>= 1;
        }
        return 0;
    }
    catch (error) {
    }
    return defInt;
}
