'use strict';

import jco = require("./jco");
import jbase_utils = require('./jbase_utils');
import jLog = require('./jlog');
var jlog = jLog(__filename);

// 运行列表
var runlist = <Array<{ name: string, type: string, cbFun: any }>>[];

function doProcess() {
    jbase_utils.seqDo("mocha_test", function* (): any {
        var doObj = runlist.shift();
        if (doObj) {
            if (doObj.type === "describe") {
                console.log("  " + doObj.name);
            }
            else {
                console.log("      " + doObj.name);
                var ret = doObj.cbFun();
                //if (Object.prototype.toString.call(ret) === "[object Promise]") {
                yield ret;
                //}
                //else {
                //    console.error("[warn]it function return is not a Promise");
                // }
            }
        }
    });
}

/**
 * 初始化mocha单元测试
 * @param global 文件的global对象
 * @returns 无
 */
export function initMochaUtils(global: any) {
    var callSelf = global.global;
    if (!callSelf.describe) {
        // node模式
        callSelf.nodeMode = true;

        // 超时函数
        callSelf.timeout = () => { };

        //慢函数
        callSelf.slow = () => { };

        // 包
        callSelf.describe = function (name: string, cbFun: any) {
            runlist.push({
                name: name,
                type: "describe",
                cbFun: cbFun
            });
            cbFun();
            setTimeout(doProcess, 0);
        }

        // 单元
        callSelf.it = function (name: string, cbFun: any) {
            runlist.push({
                name: name,
                type: "it",
                cbFun: cbFun
            });
            setTimeout(doProcess, 0);
        }
    }
}


/**
 * 退出mocha单元测试
 * @param global 文件的global对象
 * @returns 无
 */
export function exitMochaUtils(global: any) {

    // 强制退出,防止第三方插件有后台任务
    global.global.describe('Finish', function () {
        global.global.it('Exit', function () {
            setTimeout(() => {
                jlog.info("All test finish");
                console.log("All test finish");
                // if (!global.global.nodeMode) { // 仅在mocha模式才退出
                var process = require('process');
                process.exit(0);
                // }
            }, 100);
        });

    });

}