"use strict";
/*
    该模块为基础模块,不应导入任何其他内部模块
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.getString = getString;
exports.getNumber = getNumber;
exports.getInt = getInt;
exports.getBool = getBool;
exports.getArray = getArray;
exports.getDate = getDate;
exports.toDateStr = toDateStr;
exports.isYieldFun = isYieldFun;
exports.toMoneyInt = toMoneyInt;
exports.intMoneyToStr = intMoneyToStr;
exports.toMoneyStr = toMoneyStr;
var moment;
try {
    moment = require('moment');
}
catch (error) {
}
/**
 * 检测类型并获取对象
 * @param obj 对象
 * @param defVal 默认值,若规则不匹配则返回该值
 * @param mode 模式,不填=没有特殊验证,true=必须有长度,正整数为length必须等于该值
 * @returns 返回对象
 */
function getString(obj, defVal, mode) {
    if ((typeof (obj) === "string") // 字符串类型
        && ((mode === undefined || mode === 0 || (mode === false))
            || (mode === true && obj.length > 0)
            || (obj.length === mode)) // 符合mode规则
    ) {
        return obj;
    }
    else if (defVal === Error) {
        throw new Error("Check type error");
    }
    else {
        return defVal;
    }
}
/**
 * 检测类型并获取对象
 * @param obj 对象
 * @param defVal 默认值,若规则不匹配则返回该值
 * @returns 返回对象
 */
function getNumber(obj, defVal) {
    if ((typeof (obj) === 'number') && isFinite(obj)) {
        return obj;
    }
    else if (defVal === Error) {
        throw new Error("Check type error");
    }
    else {
        return defVal;
    }
}
/**
 * 检测类型并获取对象
 * @param obj 对象
 * @param defVal 默认值,若规则不匹配则返回该值
 * @param min 最小值,不填=没有验证
 * @param max 最大值,不填=没有验证
 * @returns 返回对象
 */
function getInt(obj, defVal, min, max) {
    if (Number.isInteger(obj)) { // 为整数
        if (min === undefined) {
            return obj;
        }
        else if (max === undefined) {
            if (obj >= min) {
                return obj;
            }
        }
        else if (obj >= min && obj <= max) {
            return obj;
        }
    }
    if (defVal === Error) {
        throw new Error("Check type error");
    }
    else {
        return defVal;
    }
}
/**
 * 检测类型并获取对象
 * @param obj 对象
 * @param defVal 默认值,若规则不匹配则返回该值
 * @returns 返回对象
 */
function getBool(obj, defVal) {
    if ((obj === true) || (obj === false)) {
        return obj;
    }
    else if (defVal === Error) {
        throw new Error("Check type error");
    }
    else {
        return defVal;
    }
}
/**
 * 检测类型并获取对象
 * @param obj 对象
 * @param defVal 默认值,若规则不匹配则返回该值
 * @param mode 模式,不填=没有特殊验证,正整数,成员数必须大于等于该数值
 * @returns 返回对象
 */
function getArray(obj, defVal, mode) {
    if ((Object.prototype.toString.call(obj) === '[object Array]')
        && ((mode === undefined || mode === 0)
            || (mode > 0 && obj.length >= mode)) // 符合mode规则
    ) {
        return obj;
    }
    if (defVal === Error) {
        throw new Error("Check type error");
    }
    else {
        return defVal;
    }
}
/**
 * 检测类型并获取对象
 * @param obj 对象
 * @param defVal 默认值,若规则不匹配则返回该值
 * @param mode 模式,不填=没有特殊处理,1=返回格林int戳,2="YYYY-MM-DD HH:mm:ss",3=IOS格式,"字符串自定义时间"=自定义(需要安装moment)
 * @returns 返回对象
 */
function getDate(obj, defVal, mode) {
    // 字符串时间
    if (typeof (obj) === "string") {
        try {
            obj = new Date(obj);
            if (isNaN(obj.getTime())) {
                obj = null;
            }
        }
        catch (error) {
            obj = null;
        }
    }
    // 类型判断
    if (Object.prototype.toString.call(obj) === '[object Date]') {
        if (!mode) {
            return obj;
        }
        else if (typeof (mode) === "string") {
            if (moment) {
                try {
                    var tim1 = moment(obj);
                    return tim1.format(mode);
                }
                catch (error) {
                    obj = null;
                }
            }
            else {
                throw new Error("no moment");
            }
        }
        else if (mode === 1) {
            return obj.getTime();
        }
        else if (mode === 2) {
            return obj.getFullYear() + "-" + ("00" + (obj.getMonth() + 1)).substr(-2) + "-" + ("00" + obj.getDate()).substr(-2)
                + " " + ("00" + obj.getHours()).substr(-2) + ":" + ("00" + obj.getMinutes()).substr(-2) + ":" + ("00" + obj.getSeconds()).substr(-2);
        }
        else if (mode === 3) {
            return obj.toISOString();
        }
    }
    if (defVal === Error) {
        throw new Error("Check type error");
    }
    else {
        return defVal;
    }
}
/**
 * 转换到日期字符串
 * @param obj 日期
 */
function toDateStr(obj, def) {
    if (obj) {
        return obj.getFullYear() + "-" + ("00" + (obj.getMonth() + 1)).substr(-2) + "-" + ("00" + obj.getDate()).substr(-2)
            + " " + ("00" + obj.getHours()).substr(-2) + ":" + ("00" + obj.getMinutes()).substr(-2) + ":" + ("00" + obj.getSeconds()).substr(-2);
    }
    return (def === undefined) ? obj : def;
}
// 检测函数
var yieldTestFun1 = function () {
};
var yieldTestFun2 = () => {
};
/**
 * 是否为yield可使用的函数
 * @param fun 函数
 * @returns 布尔值
 */
function isYieldFun(fun) {
    if (fun) {
        // Promise
        if (typeof (fun.then) === 'function') {
            return true;
        }
        // 非标准函数
        if (typeof (fun) === 'function') {
            return ((yieldTestFun1.__proto__ === fun.__proto__)
                || (yieldTestFun2.__proto__ === fun.__proto__)) ? false : true;
        }
    }
    return false;
}
/**
 * 金额转金额整数
 * @param val 金额
 * @returns 金额整数(单位:分)
 */
function toMoneyInt(val) {
    var valSp = (val + "").split(".");
    var negPer = (+(valSp[0] + valSp[1])) < 0 ? -1 : 1;
    var lastStr = (valSp[1] || "") + "00";
    return (((+valSp[0]) * negPer) * 100 + (+lastStr[0]) * 10 + (+lastStr[1])) * negPer;
}
/**
 * 金额整数转金额字符串
 * @param moneyInt 金额整数(单位:分)
 * @param noEndZero 去掉尾巴的0
 * @returns 金额字符串,强制保留两位小数
 */
function intMoneyToStr(moneyInt, noEndZero) {
    var moneyStr1 = ((moneyInt < 0) ? moneyInt - 100 : moneyInt + 100) + "";
    var endStr = "." + moneyStr1.substr(moneyStr1.length - 2);
    // 取出末尾的0
    if (noEndZero) {
        // 全0
        if (endStr === ".00") {
            endStr = "";
        }
        // 最后一个是0
        else if (endStr[2] === "0") {
            endStr = endStr.substring(0, 2);
        }
    }
    // 返回
    return ((moneyInt / 100) + "").split(".")[0] + endStr;
}
/**
 * 金额转金额字符串
 * @param val 金额
 * @param noEndZero 去掉尾巴的0
 * @returns 金额字符串,强制保留两位小数
 */
function toMoneyStr(val, noEndZero) {
    return intMoneyToStr(toMoneyInt(val), noEndZero);
}
