"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadLanguage = loadLanguage;
exports.getCurLng = getCurLng;
exports.t = t;
const json_utils = require("./json_utils");
const jbase_utils = require("./jbase_utils");
const fs = require("fs");
const path = require("path");
const Jlog = require("./jlog");
var jlog = Jlog("tlng");
/**
 * 语言对象数据
 */
var lngDataObj = {};
/**
 * 使用的语音名称
 */
var m_lngName = "";
/**
 * 载入语言文件
 * @param lngFileDir 语言文件路径
 * @param defLng 定义语言(在写代码时候的语言)
 * @param userLng 用户设置的语言(用户配置文件指定要看的语言)
 */
function loadLanguage(lngFileDir, defLng, userLng) {
    var lngAllStr = "{}";
    var tmp = {};
    // 载入默认语言
    {
        m_lngName = defLng || "en-US";
        try {
            lngAllStr = fs.readFileSync(path.resolve(lngFileDir + "/" + m_lngName + ".json"), "utf8");
            // 解析内容
            try {
                tmp = JSON.parse(json_utils.jsonTrim(lngAllStr));
                jbase_utils.unionObj(lngDataObj, tmp); // 合并内容
            }
            catch (error) {
                if (jlog.jlogEnable) {
                    jlog.err(`parse json error,file:${m_lngName}.json`);
                }
            }
        }
        catch (error) {
        }
    }
    // 载入用户语言
    if (userLng) {
        try {
            m_lngName = userLng;
            lngAllStr = fs.readFileSync(path.resolve(lngFileDir + "/" + userLng + ".json"), "utf8");
            // 解析内容
            try {
                tmp = JSON.parse(json_utils.jsonTrim(lngAllStr));
                jbase_utils.unionObj(lngDataObj, tmp); // 合并内容
                if (jlog.jlogEnable) {
                    jlog.info(`load language file:${userLng}.json`);
                }
            }
            catch (error) {
                if (jlog.jlogEnable) {
                    jlog.err(`parse json error,file:${userLng}.json`);
                }
            }
        }
        catch (error) {
            if (jlog.jlogEnable) {
                jlog.warn(`not find language file:${userLng}.json`);
            }
        }
    }
    return m_lngName;
}
/**
 * 获取当前语言
 * @returns 当前语言
 */
function getCurLng() {
    return m_lngName;
}
/**
 * 翻译内容
 * @param lName 语言标签名称
 * @param def 不存在时的翻译文本
 */
function t(lName, def) {
    return lngDataObj[lName] || def || lName;
}
