'use strict';

var assert = require('assert'); // 断言函数
var jutils = require('../index');

var jlog = jutils.jlog(__filename);

var JaAmqp = jutils.JaAmqp;

describe("JaAmqp", function () {

    it('base', function () {

        return jutils.jco(function* () {

            var mq = new JaAmqp();
            yield mq.init({
                "host": "127.0.0.1",
                "port": 5672,
                "login": "admin",
                "password": "admin",
                "vhost": "/"
            });

            var sub = mq.createSub();
            yield sub.init({
                // 订阅配置
                subscribe: {
                    ack: true, // 需要应答(该选项为true时,prefetchCount,autoFunAck可用)
                    prefetchCount: 10, // 预读取数量,0=无限制
                    isJsonObj: true, // 解析为json对象体,若解析失败则消息为null
                    autoFunAck: true // function*结束后自动应答
                },

                // 默认绑定的Exchange
                exchange: {
                    name: 'xxxxe_Recv_v1', // Exchange名称

                    // 如果是被动接受者则不需下列对象
                    options: { // 创建Exchange的属性
                        type: 'topic', // direct:完全匹配,topic:不完全匹配,fanout:不进行过滤
                        durable: true, // 开启持久化
                        autoDelete: false // 当删除了所有的queue时自动删除Exchange
                    }
                },

                // 通道配置
                queue: {
                    name: 'xxxxq_Recv_v1', // Queue名称

                    // 默认绑定的routekey
                    routing: 'api_v1',

                    // <可省>绑定其他Exchange和routing
                    bindEx: [
                        {
                            exchange: "my_test_ex",
                            routing: "#"
                        }
                    ],

                    // <可省>如果是被动使用共享的Queue则不需下列对象
                    options: {
                        durable: true, // 开启持久化
                        autoDelete: false // 当连接全部断开时自动删除Queue
                    }

                }
            }, function* (inMsg, headers, deliveryInfo, messageObject, errTryCount) {
                console.log(inMsg);

            });


            var pub = mq.createPub();
            yield pub.init({
                exchange: {
                    name: "my_test_ex",
                    // 如果是被动接受者则不需下列对象
                    options: { // 创建Exchange的属性
                        type: 'topic', // direct:完全匹配,topic:不完全匹配,fanout:不进行过滤
                        durable: true, // 开启持久化
                        autoDelete: false // 当删除了所有的queue时自动删除Exchange
                    }
                }

            });

            var xx = 1;
            setInterval(() => {
                pub.publishMsg({ a: "abc" + (xx++) }, "abcd");
            }, 1000);

        });
    });


    // it('ab', function () {

    //     return jutils.jco(function*() {

    //         var count_1=0;
    //         var count_2=0;

    //         var mq = new JaAmqp();
    //         yield mq.init({
    //             "host": "127.0.0.1",
    //             "port": 5672,
    //             "login": "guest",
    //             "password": "guest",
    //             "vhost": "/"
    //         });

    //         var sub = mq.createSub();
    //         yield sub.init({
    //             // 订阅配置
    //             subscribe: {
    //                 ack: true, // 需要应答(该选项为true时,prefetchCount,autoFunAck可用)
    //                 prefetchCount: 10, // 预读取数量,0=无限制
    //                 isJsonObj: true, // 解析为json对象体,若解析失败则消息为null
    //                 autoFunAck: true // function*结束后自动应答
    //             },

    //             // 默认绑定的Exchange
    //             exchange: {
    //                 name: 'xxxxe_Recv_v1', // Exchange名称

    //                 // 如果是被动接受者则不需下列对象
    //                 options: { // 创建Exchange的属性
    //                     type: 'direct', // direct:完全匹配,topic:不完全匹配,fanout:不进行过滤
    //                     durable: true, // 开启持久化
    //                     autoDelete: false // 当删除了所有的queue时自动删除Exchange
    //                 }
    //             },

    //             // 通道配置
    //             queue: {
    //                 name: 'xxxxq_Recv_v1', // Queue名称

    //                 // 默认绑定的routekey
    //                 routing: 'api_v1',

    //                 // <可省>绑定其他Exchange和routing
    //                 bindEx: [
    //                     {
    //                         exchange: "amq.topic",
    //                         routing: "#"
    //                     }
    //                 ],

    //                 // <可省>如果是被动使用共享的Queue则不需下列对象
    //                 options: {
    //                     durable: true, // 开启持久化
    //                     autoDelete: false // 当连接全部断开时自动删除Queue
    //                 }

    //             }
    //         }, function* (inMsg, headers, deliveryInfo, messageObject, errTryCount) {
    //             count_1++;
    //         });


    //         var pub = mq.createPub();
    //         yield pub.init({
    //             exchange:{
    //                 name:"amq.topic"
    //             }
    //         });

    //         jlog.setEnable(false,"*");

    //         var tt = setInterval(() => {
    //             pub.publishMsg({ a: "11111111111111111" }, "abcd").then((x)=>{

    //             }).catch((err)=>{
    //                 console.log("eeeeeeeeeeeeeeeeeeeeee");

    //             });
    //             count_2++;
    //             if(count_2>=10000){
    //                 clearInterval(tt);
    //             }
    //         }, 1); 


    //         setInterval(() => {
    //             console.log("count_1="+count_1+",count_2="+count_2);
    //         }, 1000); 

    //     });
    // });

});

