'use strict';

var assert = require('assert'); // 断言函数

var jutils = require('../index');

describe('Redis', function () { // 测试目录层(可以套任意层)

    it('JaRedisLocal', function () { // 测试用例
        return jutils.jco(function*() {

            var myRedis = new jutils.JaRedisLocal();
            //var myRedis = new jutils.JaRedis();

            yield myRedis.init({
                //"host": "./temp/redis.json" // 文件
                //"saveTimeUp":100 // 存储间隔
                "host": "127.0.0.1", // IP
                "port": 6379, // 端口
                //"count": 5, // 连接数,默认:1
                "database": 0, // 数据库ID,0~15
                "keyfirst": "device_gateway", // key头字符串
                "password": undefined // 密码
            });

            yield myRedis.del("a1"); // 清理
            yield myRedis.del("ah1"); // 清理
            yield myRedis.del("a2"); // 清理
            yield myRedis.del("a3"); // 清理

            var x1 = yield myRedis.set("a1", 456, 10);
            var x2 = yield myRedis.get("a1");
            var xh1 = yield myRedis.hset("ah1", "f1", 789);
            var xht1 = yield myRedis.expire("ah1", 2);
            var xh2 = yield myRedis.hget("ah1", "f1");

            if (x1 !== "OK" || x2 !== "456" || xh1 !== 1 || xh2 !== "789") {
                assert.fail("JaRedis");
            }
            
            var x6 = yield myRedis.getLockWait("a3", 5, 0);
            var x7 = yield myRedis.getLockWait("a3", 100, 0);
            var x8 = yield myRedis.del("a3");
            var x9 = yield myRedis.getLockWait("a3", 100, 0);

            if (x6 !== "OK" || x7 || x8 !== 1 || x9 !== "OK") {
                assert.fail("JaRedis");
            }

        });
    });

    it('JaRedis', function () { // 测试用例
  
       return jutils.jco(function*() {

            var JaRedis = jutils.JaRedis;
            var myRedis = new JaRedis();

            yield myRedis.init({
                "host": "127.0.0.1", // IP
                "port": 6379, // 端口
                "count": 5, // 连接数,默认:5
                "database": 0, // 数据库ID,0~15
                "keyfirst": "jtest", // key头字符串
                "password": undefined // 密码
            });
            // var client = require('redis').createClient();
            // yield myRedis.init([client]);

            yield myRedis.del("a1"); // 清理
            yield myRedis.del("ah1"); // 清理
            yield myRedis.del("a2"); // 清理
            yield myRedis.del("a3"); // 清理

            var x1 = yield myRedis.set("a1", 456, 10);
            var x2 = yield myRedis.get("a1");
            var xh1 = yield myRedis.hset("ah1", "f1", 789);
            var xh2 = yield myRedis.hget("ah1", "f1");

            var x3 = yield myRedis.llpush("a2", "bbbb");
            var x4 = yield myRedis.lrpop("a2", 10);

            var x6 = yield myRedis.getLockWait("a3", 100, 0);
            var x7 = yield myRedis.getLockWait("a3", 100, 0);
            var x8 = yield myRedis.del("a3");
            var x9 = yield myRedis.getLockWait("a3", 100, 0);

            if (x1 !== "OK" || x2 !== "456" || xh1 !== 1 || xh2 !== "789" || typeof (x3) !== "number" || x4 !== "bbbb" || x6 !== "OK" || x7 || x8 !== 1 || x9 !== "OK") {
                assert.fail("JaRedis");
            }

        });
    });

    // it('JaIoRedis', function () { // 测试用例
    //      return jutils.jco(function*() {
    //         var JaIoRedis = jutils.JaIoRedis;
    //         var myRedis = new JaIoRedis();

    //         // yield myRedis.init({
    //         //     "host": "127.0.0.1", // IP
    //         //     "port": 6379, // 端口
    //         //     "count": 5, // 连接数,默认:5
    //         //     "database": 0, // 数据库ID,0~15
    //         //     "keyfirst": "device_gateway", // key头字符串
    //         //     "password": undefined // 密码
    //         // });
    //         var client = require('ioredis')();
    //         yield myRedis.init([client]);

    //         yield myRedis.del("a1"); // 清理
    //         yield myRedis.del("ah1"); // 清理
    //         yield myRedis.del("a2"); // 清理
    //         yield myRedis.del("a3"); // 清理

    //         var x1 = yield myRedis.set("a1", 456, 10);
    //         var x2 = yield myRedis.get("a1");
    //         var xh1 = yield myRedis.hset("ah1", "f1", 789);
    //         var xh2 = yield myRedis.hget("ah1", "f1");

    //         var x3 = yield myRedis.llpush("a2", "bbbb");
    //         var x4 = yield myRedis.lrpop("a2", 10);

    //         var x6 = yield myRedis.getLockWait("a3", 100, 0);
    //         var x7 = yield myRedis.getLockWait("a3", 100, 0);
    //         var x8 = yield myRedis.del("a3");
    //         var x9 = yield myRedis.getLockWait("a3", 100, 0);

    //         // if (x1 !== "OK" || x2 !== "456" || xh1 !== 1 || xh2 !== "789" || typeof (x3) !== "number" || x4 !== "bbbb" || x6 !== "OK" || x7 || x8 !== 1 || x9 !== "OK") {
    //         //     assert.fail("JaRedis");
    //         // }


    //     });
    // });

});
