'use strict';

var assert = require('assert'); // 断言函数
var jutils = require('../index');
var jlog = jutils.jlog(__filename);

var jsig_utils = jutils.jsig_utils;

describe("jsig_utils", function () {
    it('jsig_wait', function () {

        return jutils.jco(function* () {
            // 测试阻塞确认
            var testBlock = yield jsig_utils.sigWait(-1, "a123", function* () {
                jsig_utils.sigAck("a123", { success: "OK" });
            });
            jlog.info(testBlock);

            // 测试演示确认
            setTimeout(() => {
                jsig_utils.sigAck(123, "abc");
            }, 3000);

            jutils.jco(function* () {
                var ret = yield jsig_utils.sigWait(10000, 123, function* () {
                    //jsig_utils.sigAck(123,"def");
                });
                jlog.info("sig wait ok1:" + ret);
                assert(ret === undefined);
            });

            jutils.jco(function* () {
                var ret = yield jsig_utils.sigWait(10000, 123, function* () {
                    //jsig_utils.sigAck(123,"def");
                });
                jlog.info("sig wait ok2:" + ret);
                assert(ret === "abc");
            });


            jutils.jco(function* () {
                var ret = yield jsig_utils.sigWait(10000, 456, function* () {
                    jsig_utils.sigAck(456, "def");
                });
                jlog.info("sig wait ok3:" + ret);
                assert(ret === "def");
            });


            jutils.jco(function* () {
                var ret = yield jsig_utils.sigWait(10000, 4561, function* () {
                    jsig_utils.sigAck(4561, null);
                });
                jlog.info("sig wait ok4:" + ret);
                assert(ret === null);
            });
        });
    });

    it('jsig_env', function () {

        return jutils.jco(function* () {

            // jlog.info("env wait");
            // var ret1 = yield jsig_utils.envWait("wait1",5000);
            // jlog.info("env wait no:"+ret1);

            // setTimeout(() => {
            //     jsig_utils.envSet("wait1",true);
            // }, 2000);

            // jsig_utils.envSet("wait1",false);

            // for (var index = 0; index < 5; index++) {

            //      return jutils.jco(function*() {
            //         var ret = yield jsig_utils.envWait("wait1",5000);
            //         jlog.info("env wait ok:"+ret);
            //     });
            // }

            // var ret2 = yield jsig_utils.envWait("wait1",-1);
            // jlog.info("env wait main:"+ret2);


            var taskPromise = jsig_utils.taskListStart("myTask", 2);

            var testFun = function* (a, b) {
                jlog.info(a);
                yield jutils.sleep(b);
            }

            setTimeout(() => {
                jsig_utils.taskListAdd("myTask", {
                    fun: jlog.info,
                    args: [">>>>1"]
                });
                jsig_utils.taskListAdd("myTask", {
                    name: "aaa",
                    fun: jlog.info,
                    args: [">>>>2"]
                });
                jsig_utils.taskListAdd("myTask", {
                    name: "aaa",
                    fun: jlog.info,
                    args: [">>>>3"]
                });


                var tList = [
                    {
                        name: "aaa",
                        fun: jlog.info,
                        args: [">>>>4"]
                    },
                    {
                        fun: jlog.info,
                        groupId: 111111,
                        args: [">>>>5{1}"]
                    },
                    {
                        fun: jlog.info,
                        args: [">>>>6"]
                    },
                    {
                        fun: testFun,
                        args: [">>>>7", 1000]
                    },
                    {
                        fun: testFun,
                        args: [">>>>8", 1000]
                    },
                    {
                        fun: testFun,
                        args: [">>>>9", 1000]
                    },
                    {
                        fun: testFun,
                        args: [">>>>10", 1000]
                    },
                    {
                        fun: testFun,
                        args: [">>>>11", 1000]
                    },
                    {
                        fun: testFun,
                        args: [">>>>12", 1000]
                    },
                    {
                        fun: testFun,
                        args: [">>>>13", 1000]
                    }];

                // 
                //tList.length=jutils.getRndInt(1,4)

                jsig_utils.taskListAdd("myTask", tList);
                //jsig_utils.taskListDel("myTask","aaa")
            }, 2000);

            setTimeout(() => {
                jsig_utils.taskListStop("myTask", false);
            }, 11000);

            yield taskPromise;

            jlog.info("finish");



        });
    });
});

